@@ -26,6 +26,7 @@ import (
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/asserts"
+	"github.com/snapcore/snapd/asserts/assertstest"
 )
 
 type modelSuite struct {
@@ -209,7 +210,7 @@ func (mods *modelSuite) TestModelCheck(c *C) {
 	c.Assert(err, IsNil)
 
 	storeDB, db := makeStoreAndCheckDB(c)
-	brandDB := setup3rdPartySigning(c, "brand1", storeDB, db)
+	brandDB := setup3rdPartySigning(c, "brand-id1", storeDB, db)
 
 	headers := ex.Headers()
 	headers["brand-id"] = brandDB.AuthorityID
@@ -226,7 +227,7 @@ func (mods *modelSuite) TestModelCheckInconsistentTimestamp(c *C) {
 	c.Assert(err, IsNil)
 
 	storeDB, db := makeStoreAndCheckDB(c)
-	brandDB := setup3rdPartySigning(c, "brand1", storeDB, db)
+	brandDB := setup3rdPartySigning(c, "brand-id1", storeDB, db)
 
 	headers := ex.Headers()
 	headers["brand-id"] = brandDB.AuthorityID
@@ -350,6 +351,7 @@ func (ss *serialSuite) TestDecodeInvalid(c *C) {
 		{"authority-id: brand-id1\n", "authority-id: random\n", `authority-id and brand-id must match, serial assertions are expected to be signed by the brand: "random" != "brand-id1"`},
 		{"model: baz-3000\n", "", `"model" header is mandatory`},
 		{"model: baz-3000\n", "model: \n", `"model" header should not be empty`},
+		{"model: baz-3000\n", "model: _what\n", `"model" header contains invalid characters: "_what"`},
 		{"serial: 2700\n", "", `"serial" header is mandatory`},
 		{"serial: 2700\n", "serial: \n", `"serial" header should not be empty`},
 		{ss.tsLine, "", `"timestamp" header is mandatory`},
@@ -379,6 +381,42 @@ func (ss *serialSuite) TestDecodeKeyIDMismatch(c *C) {
 	c.Check(err, ErrorMatches, serialErrPrefix+"device key does not match provided key id")
 }
 
+func (ss *serialSuite) TestSerialCheck(c *C) {
+	encoded := strings.Replace(serialExample, "TSLINE", ss.tsLine, 1)
+	encoded = strings.Replace(encoded, "DEVICEKEY", strings.Replace(ss.encodedDevKey, "\n", "\n    ", -1), 1)
+	encoded = strings.Replace(encoded, "KEYID", ss.deviceKey.PublicKey().ID(), 1)
+	ex, err := asserts.Decode([]byte(encoded))
+	c.Assert(err, IsNil)
+
+	storeDB, db := makeStoreAndCheckDB(c)
+	brandDB := setup3rdPartySigning(c, "brand1", storeDB, db)
+
+	tests := []struct {
+		signDB  assertstest.SignerDB
+		brandID string
+		authID  string
+		keyID   string
+	}{
+		{brandDB, brandDB.AuthorityID, "", brandDB.KeyID},
+	}
+
+	for _, test := range tests {
+		headers := ex.Headers()
+		headers["brand-id"] = test.brandID
+		if test.authID != "" {
+			headers["authority-id"] = test.authID
+		} else {
+			headers["authority-id"] = test.brandID
+		}
+		headers["timestamp"] = time.Now().Format(time.RFC3339)
+		serial, err := test.signDB.Sign(asserts.SerialType, headers, nil, test.keyID)
+		c.Assert(err, IsNil)
+
+		err = db.Check(serial)
+		c.Check(err, IsNil)
+	}
+}
+
 func (ss *serialSuite) TestSerialRequestHappy(c *C) {
 	sreq, err := asserts.SignWithoutAuthority(asserts.SerialRequestType,
 		map[string]interface{}{
