@@ -26,7 +26,6 @@ import (
 	"mime/multipart"
 	"net/http"
 	"net/http/httptest"
-	"os"
 	"path/filepath"
 	"regexp"
 	"time"
@@ -35,6 +34,9 @@ import (
 
 	"github.com/snapcore/snapd/client"
 	snap "github.com/snapcore/snapd/cmd/snap"
+	"github.com/snapcore/snapd/progress"
+	"github.com/snapcore/snapd/progress/progresstest"
+	"github.com/snapcore/snapd/testutil"
 )
 
 type snapOpTestServer struct {
@@ -103,6 +105,8 @@ func (s *SnapOpSuite) TearDownTest(c *check.C) {
 }
 
 func (s *SnapOpSuite) TestWait(c *check.C) {
+	meter := &progresstest.Meter{}
+	defer progress.MockMeter(meter)()
 	restore := snap.MockMaxGoneTime(time.Millisecond)
 	defer restore()
 
@@ -111,27 +115,16 @@ func (s *SnapOpSuite) TestWait(c *check.C) {
 	snap.ClientConfig.BaseURL = server.URL
 	server.Close()
 
-	d := c.MkDir()
-	oldStdout := os.Stdout
-	stdout, err := ioutil.TempFile(d, "stdout")
-	c.Assert(err, check.IsNil)
-	defer func() {
-		os.Stdout = oldStdout
-		stdout.Close()
-		os.Remove(stdout.Name())
-	}()
-	os.Stdout = stdout
-
 	cli := snap.Client()
 	chg, err := snap.Wait(cli, "x")
 	c.Assert(chg, check.IsNil)
 	c.Assert(err, check.NotNil)
-	buf, err := ioutil.ReadFile(stdout.Name())
-	c.Assert(err, check.IsNil)
-	c.Check(string(buf), check.Matches, "(?ms).*Waiting for server to restart.*")
+	c.Check(meter.Labels, testutil.Contains, "Waiting for server to restart")
 }
 
 func (s *SnapOpSuite) TestWaitRecovers(c *check.C) {
+	meter := &progresstest.Meter{}
+	defer progress.MockMeter(meter)()
 	restore := snap.MockMaxGoneTime(time.Millisecond)
 	defer restore()
 
@@ -144,27 +137,14 @@ func (s *SnapOpSuite) TestWaitRecovers(c *check.C) {
 		fmt.Fprintln(w, `{"type": "sync", "result": {"ready": true, "status": "Done"}}`)
 	})
 
-	d := c.MkDir()
-	oldStdout := os.Stdout
-	stdout, err := ioutil.TempFile(d, "stdout")
-	c.Assert(err, check.IsNil)
-	defer func() {
-		os.Stdout = oldStdout
-		stdout.Close()
-		os.Remove(stdout.Name())
-	}()
-	os.Stdout = stdout
-
 	cli := snap.Client()
 	chg, err := snap.Wait(cli, "x")
 	// we got the change
 	c.Assert(chg, check.NotNil)
 	c.Assert(err, check.IsNil)
-	buf, err := ioutil.ReadFile(stdout.Name())
-	c.Assert(err, check.IsNil)
 
 	// but only after recovering
-	c.Check(string(buf), check.Matches, "(?ms).*Waiting for server to restart.*")
+	c.Check(meter.Labels, testutil.Contains, "Waiting for server to restart")
 }
 
 func (s *SnapOpSuite) TestInstall(c *check.C) {
@@ -443,6 +423,29 @@ func (s *SnapOpSuite) TestInstallPathDangerous(c *check.C) {
 	c.Check(s.srv.n, check.Equals, s.srv.total)
 }
 
+func (s *SnapOpSuite) TestRevertRunthrough(c *check.C) {
+	s.srv.total = 4
+	s.srv.channel = "potato"
+	s.srv.checker = func(r *http.Request) {
+		c.Check(r.URL.Path, check.Equals, "/v2/snaps/foo")
+		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
+			"action": "revert",
+		})
+	}
+
+	s.RedirectClientToTestServer(s.srv.handle)
+	rest, err := snap.Parser().ParseArgs([]string{"revert", "foo"})
+	c.Assert(err, check.IsNil)
+	c.Assert(rest, check.DeepEquals, []string{})
+	// tracking channel is "" in the test server
+	c.Check(s.Stdout(), check.Equals, `foo reverted to 1.0
+This leaves foo tracking .
+`)
+	c.Check(s.Stderr(), check.Equals, "")
+	// ensure that the fake server api was actually hit
+	c.Check(s.srv.n, check.Equals, s.srv.total)
+}
+
 func (s *SnapOpSuite) runRevertTest(c *check.C, opts *client.SnapOptions) {
 	modes := []struct {
 		enabled bool
@@ -996,8 +999,8 @@ func (s *SnapOpSuite) TestNoWait(c *check.C) {
 		{"try", "--no-wait", "."},
 	}
 
+	s.RedirectClientToTestServer(s.srv.handle)
 	for _, cmd := range cmds {
-		s.RedirectClientToTestServer(s.srv.handle)
 		rest, err := snap.Parser().ParseArgs(cmd)
 		c.Assert(err, check.IsNil, check.Commentf("%v", cmd))
 		c.Assert(rest, check.DeepEquals, []string{})
@@ -1009,3 +1012,33 @@ func (s *SnapOpSuite) TestNoWait(c *check.C) {
 		s.stdout.Reset()
 	}
 }
+
+func (s *SnapOpSuite) TestSwitchHappy(c *check.C) {
+	s.srv.total = 3
+	s.srv.checker = func(r *http.Request) {
+		c.Check(r.URL.Path, check.Equals, "/v2/snaps/foo")
+		c.Check(DecodedRequestBody(c, r), check.DeepEquals, map[string]interface{}{
+			"action":  "switch",
+			"channel": "beta",
+		})
+	}
+
+	s.RedirectClientToTestServer(s.srv.handle)
+	rest, err := snap.Parser().ParseArgs([]string{"switch", "--beta", "foo"})
+	c.Assert(err, check.IsNil)
+	c.Assert(rest, check.DeepEquals, []string{})
+	c.Check(s.Stdout(), check.Matches, `(?sm).*"foo" switched to the "beta" channel`)
+	c.Check(s.Stderr(), check.Equals, "")
+	// ensure that the fake server api was actually hit
+	c.Check(s.srv.n, check.Equals, s.srv.total)
+}
+
+func (s *SnapOpSuite) TestSwitchUnhappy(c *check.C) {
+	_, err := snap.Parser().ParseArgs([]string{"switch"})
+	c.Assert(err, check.ErrorMatches, "the required argument `<snap>` was not provided")
+}
+
+func (s *SnapOpSuite) TestSwitchAlsoUnhappy(c *check.C) {
+	_, err := snap.Parser().ParseArgs([]string{"switch", "foo"})
+	c.Assert(err, check.ErrorMatches, `missing --channel=<channel-name> parameter`)
+}
