@@ -44,8 +44,9 @@ import (
 	"github.com/snapcore/snapd/asserts/snapasserts"
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/dirs"
-	"github.com/snapcore/snapd/i18n/dumb"
+	"github.com/snapcore/snapd/i18n"
 	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/jsonutil"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord/assertstate"
@@ -55,6 +56,7 @@ import (
 	"github.com/snapcore/snapd/overlord/devicestate"
 	"github.com/snapcore/snapd/overlord/hookstate/ctlcmd"
 	"github.com/snapcore/snapd/overlord/ifacestate"
+	"github.com/snapcore/snapd/overlord/servicestate"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/progress"
@@ -62,6 +64,7 @@ import (
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/store"
 	"github.com/snapcore/snapd/strutil"
+	"github.com/snapcore/snapd/systemd"
 )
 
 var api = []*Command{
@@ -86,6 +89,8 @@ var api = []*Command{
 	usersCmd,
 	sectionsCmd,
 	aliasesCmd,
+	appsCmd,
+	logsCmd,
 	debugCmd,
 }
 
@@ -103,8 +108,9 @@ var (
 	}
 
 	loginCmd = &Command{
-		Path: "/v2/login",
-		POST: loginUser,
+		Path:     "/v2/login",
+		POST:     loginUser,
+		PolkitOK: "io.snapcraft.snapd.login",
 	}
 
 	logoutCmd = &Command{
@@ -126,17 +132,32 @@ var (
 	}
 
 	snapsCmd = &Command{
-		Path:   "/v2/snaps",
-		UserOK: true,
-		GET:    getSnapsInfo,
-		POST:   postSnaps,
+		Path:     "/v2/snaps",
+		UserOK:   true,
+		PolkitOK: "io.snapcraft.snapd.manage",
+		GET:      getSnapsInfo,
+		POST:     postSnaps,
 	}
 
 	snapCmd = &Command{
-		Path:   "/v2/snaps/{name}",
+		Path:     "/v2/snaps/{name}",
+		UserOK:   true,
+		PolkitOK: "io.snapcraft.snapd.manage",
+		GET:      getSnapInfo,
+		POST:     postSnap,
+	}
+
+	appsCmd = &Command{
+		Path:   "/v2/apps",
 		UserOK: true,
-		GET:    getSnapInfo,
-		POST:   postSnap,
+		GET:    getAppsInfo,
+		POST:   postApps,
+	}
+
+	logsCmd = &Command{
+		Path:     "/v2/logs",
+		PolkitOK: "io.snapcraft.snapd.manage",
+		GET:      getLogs,
 	}
 
 	snapConfCmd = &Command{
@@ -148,14 +169,16 @@ var (
 	interfacesCmd = &Command{
 		Path:   "/v2/interfaces",
 		UserOK: true,
-		GET:    getInterfaces,
+		GET:    interfacesConnectionsMultiplexer,
 		POST:   changeInterfaces,
 	}
 
 	// TODO: allow to post assertions for UserOK? they are verified anyway
 	assertsCmd = &Command{
-		Path: "/v2/assertions",
-		POST: doAssert,
+		Path:   "/v2/assertions",
+		UserOK: true,
+		GET:    getAssertTypeNames,
+		POST:   doAssert,
 	}
 
 	assertsFindManyCmd = &Command{
@@ -165,10 +188,11 @@ var (
 	}
 
 	stateChangeCmd = &Command{
-		Path:   "/v2/changes/{id}",
-		UserOK: true,
-		GET:    getChange,
-		POST:   abortChange,
+		Path:     "/v2/changes/{id}",
+		UserOK:   true,
+		PolkitOK: "io.snapcraft.snapd.manage",
+		GET:      getChange,
+		POST:     abortChange,
 	}
 
 	stateChangesCmd = &Command{
@@ -243,7 +267,10 @@ func sysInfo(c *Command, r *http.Request, user *auth.UserState) Response {
 	st.Lock()
 	nextRefresh := snapMgr.NextRefresh()
 	lastRefresh, _ := snapMgr.LastRefresh()
-	refreshScheduleStr := snapMgr.RefreshSchedule()
+	refreshScheduleStr, err := snapMgr.RefreshSchedule()
+	if err != nil {
+		return InternalError("cannot get refresh schedule: %s", err)
+	}
 	users, err := auth.Users(st)
 	st.Unlock()
 	if err != nil && err != state.ErrNoState {
@@ -461,7 +488,7 @@ func getSnapInfo(c *Command, r *http.Request, user *auth.UserState) Response {
 	about, err := localSnapInfo(c.d.overlord.State(), name)
 	if err != nil {
 		if err == errNoSnap {
-			return SnapNotFound(err)
+			return SnapNotFound(name, err)
 		}
 
 		return InternalError("%v", err)
@@ -482,21 +509,17 @@ func getSnapInfo(c *Command, r *http.Request, user *auth.UserState) Response {
 	return SyncResponse(result, nil)
 }
 
-func webify(result map[string]interface{}, resource string) map[string]interface{} {
-	result["resource"] = resource
-
-	icon, ok := result["icon"].(string)
-	if !ok || icon == "" || strings.HasPrefix(icon, "http") {
+func webify(result *client.Snap, resource string) *client.Snap {
+	if result.Icon == "" || strings.HasPrefix(result.Icon, "http") {
 		return result
 	}
-	result["icon"] = ""
+	result.Icon = ""
 
 	route := appIconCmd.d.router.Get(appIconCmd.Path)
 	if route != nil {
-		name, _ := result["name"].(string)
-		url, err := route.URL("name", name)
+		url, err := route.URL("name", result.Name)
 		if err == nil {
-			result["icon"] = url.String()
+			result.Icon = url.String()
 		}
 	}
 
@@ -525,7 +548,7 @@ func getSections(c *Command, r *http.Request, user *auth.UserState) Response {
 		// pass
 	case store.ErrEmptyQuery, store.ErrBadQuery:
 		return BadRequest("%v", err)
-	case store.ErrUnauthenticated:
+	case store.ErrUnauthenticated, store.ErrInvalidCredentials:
 		return Unauthorized("%v", err)
 	default:
 		return InternalError("%v", err)
@@ -586,7 +609,7 @@ func searchStore(c *Command, r *http.Request, user *auth.UserState) Response {
 		// pass
 	case store.ErrEmptyQuery, store.ErrBadQuery:
 		return BadRequest("%v", err)
-	case store.ErrUnauthenticated:
+	case store.ErrUnauthenticated, store.ErrInvalidCredentials:
 		return Unauthorized(err.Error())
 	default:
 		return InternalError("%v", err)
@@ -611,10 +634,14 @@ func findOne(c *Command, r *http.Request, user *auth.UserState, name string) Res
 		AnyChannel: true,
 	}
 	snapInfo, err := theStore.SnapInfo(spec, user)
-	if err != nil {
-		if err == store.ErrSnapNotFound {
-			return SnapNotFound(err)
-		}
+	switch err {
+	case nil:
+		// pass
+	case store.ErrInvalidCredentials:
+		return Unauthorized("%v", err)
+	case store.ErrSnapNotFound:
+		return SnapNotFound(name, err)
+	default:
 		return InternalError("%v", err)
 	}
 
@@ -776,7 +803,7 @@ func (*licenseData) Error() string {
 }
 
 type snapInstruction struct {
-	progress.NullProgress
+	progress.NullMeter
 	Action           string        `json:"action"`
 	Channel          string        `json:"channel"`
 	Revision         snap.Revision `json:"revision"`
@@ -811,7 +838,6 @@ func (inst *snapInstruction) installFlags() (snapstate.Flags, error) {
 }
 
 var (
-	snapstateCoreInfo          = snapstate.CoreInfo
 	snapstateInstall           = snapstate.Install
 	snapstateInstallPath       = snapstate.InstallPath
 	snapstateRefreshCandidates = snapstate.RefreshCandidates
@@ -822,6 +848,7 @@ var (
 	snapstateRemoveMany        = snapstate.RemoveMany
 	snapstateRevert            = snapstate.Revert
 	snapstateRevertToRevision  = snapstate.RevertToRevision
+	snapstateSwitch            = snapstate.Switch
 
 	assertstateRefreshSnapDeclarations = assertstate.RefreshSnapDeclarations
 )
@@ -834,42 +861,6 @@ var ensureStateSoon = ensureStateSoonImpl
 
 var errNothingToInstall = errors.New("nothing to install")
 
-const oldDefaultSnapCoreName = "ubuntu-core"
-const defaultCoreSnapName = "core"
-
-func ensureCore(st *state.State, targetSnap string, userID int) (*state.TaskSet, error) {
-	if targetSnap == defaultCoreSnapName || targetSnap == oldDefaultSnapCoreName {
-		return nil, errNothingToInstall
-	}
-
-	_, err := snapstateCoreInfo(st)
-	if err != state.ErrNoState {
-		return nil, err
-	}
-
-	return snapstateInstall(st, defaultCoreSnapName, "stable", snap.R(0), userID, snapstate.Flags{})
-}
-
-func withEnsureCore(st *state.State, targetSnap string, userID int, install func() (*state.TaskSet, error)) ([]*state.TaskSet, error) {
-	ubuCoreTs, err := ensureCore(st, targetSnap, userID)
-	if err != nil && err != errNothingToInstall {
-		return nil, err
-	}
-
-	ts, err := install()
-	if err != nil {
-		return nil, err
-	}
-
-	// ensure main install waits on ubuntu core install
-	if ubuCoreTs != nil {
-		ts.WaitAll(ubuCoreTs)
-		return []*state.TaskSet{ubuCoreTs, ts}, nil
-	}
-
-	return []*state.TaskSet{ts}, nil
-}
-
 var errDevJailModeConflict = errors.New("cannot use devmode and jailmode flags together")
 var errClassicDevmodeConflict = errors.New("cannot use classic and devmode flags together")
 var errNoJailMode = errors.New("this system cannot honour the jailmode flag")
@@ -967,11 +958,7 @@ func snapInstall(inst *snapInstruction, st *state.State) (string, []*state.TaskS
 
 	logger.Noticef("Installing snap %q revision %s", inst.Snaps[0], inst.Revision)
 
-	tsets, err := withEnsureCore(st, inst.Snaps[0], inst.userID,
-		func() (*state.TaskSet, error) {
-			return snapstateInstall(st, inst.Snaps[0], inst.Channel, inst.Revision, inst.userID, flags)
-		},
-	)
+	tset, err := snapstateInstall(st, inst.Snaps[0], inst.Channel, inst.Revision, inst.userID, flags)
 	if err != nil {
 		return "", nil, err
 	}
@@ -980,7 +967,7 @@ func snapInstall(inst *snapInstruction, st *state.State) (string, []*state.TaskS
 	if inst.Channel != "stable" && inst.Channel != "" {
 		msg = fmt.Sprintf(i18n.G("Install %q snap from %q channel"), inst.Snaps[0], inst.Channel)
 	}
-	return msg, tsets, nil
+	return msg, []*state.TaskSet{tset}, nil
 }
 
 func snapUpdate(inst *snapInstruction, st *state.State) (string, []*state.TaskSet, error) {
@@ -1088,6 +1075,19 @@ func snapDisable(inst *snapInstruction, st *state.State) (string, []*state.TaskS
 	return msg, []*state.TaskSet{ts}, nil
 }
 
+func snapSwitch(inst *snapInstruction, st *state.State) (string, []*state.TaskSet, error) {
+	if !inst.Revision.Unset() {
+		return "", nil, errors.New("switch takes no revision")
+	}
+	ts, err := snapstate.Switch(st, inst.Snaps[0], inst.Channel)
+	if err != nil {
+		return "", nil, err
+	}
+
+	msg := fmt.Sprintf(i18n.G("Switch %q snap to %s"), inst.Snaps[0], inst.Channel)
+	return msg, []*state.TaskSet{ts}, nil
+}
+
 type snapActionFunc func(*snapInstruction, *state.State) (string, []*state.TaskSet, error)
 
 var snapInstructionDispTable = map[string]snapActionFunc{
@@ -1097,6 +1097,7 @@ var snapInstructionDispTable = map[string]snapActionFunc{
 	"revert":  snapRevert,
 	"enable":  snapEnable,
 	"disable": snapDisable,
+	"switch":  snapSwitch,
 }
 
 func (inst *snapInstruction) dispatch() snapActionFunc {
@@ -1111,7 +1112,7 @@ func (inst *snapInstruction) errToResponse(err error) Response {
 
 	switch err {
 	case store.ErrSnapNotFound:
-		return SnapNotFound(err)
+		return SnapNotFound(inst.Snaps[0], err)
 	case store.ErrNoUpdateAvailable:
 		kind = errorKindSnapNoUpdateAvailable
 	case store.ErrLocalSnap:
@@ -1225,21 +1226,13 @@ func trySnap(c *Command, r *http.Request, user *auth.UserState, trydir string, f
 		return BadRequest("cannot read snap info for %s: %s", trydir, err)
 	}
 
-	var userID int
-	if user != nil {
-		userID = user.ID
-	}
-	tsets, err := withEnsureCore(st, info.Name(), userID,
-		func() (*state.TaskSet, error) {
-			return snapstateTryPath(st, info.Name(), trydir, flags)
-		},
-	)
+	tset, err := snapstateTryPath(st, info.Name(), trydir, flags)
 	if err != nil {
 		return BadRequest("cannot try %s: %s", trydir, err)
 	}
 
 	msg := fmt.Sprintf(i18n.G("Try %q snap from %s"), info.Name(), trydir)
-	chg := newChange(st, "try-snap", msg, tsets, []string{info.Name()})
+	chg := newChange(st, "try-snap", msg, []*state.TaskSet{tset}, []string{info.Name()})
 	chg.Set("api-data", map[string]string{"snap-name": info.Name()})
 
 	ensureStateSoon(st)
@@ -1415,11 +1408,11 @@ out:
 
 	if !dangerousOK {
 		si, err := snapasserts.DeriveSideInfo(tempPath, assertstate.DB(st))
-		switch err {
-		case nil:
+		switch {
+		case err == nil:
 			snapName = si.RealName
 			sideInfo = si
-		case asserts.ErrNotFound:
+		case asserts.IsNotFound(err):
 			// with devmode we try to find assertions but it's ok
 			// if they are not there (implies --dangerous)
 			if !isTrue(form, "devmode") {
@@ -1450,21 +1443,12 @@ out:
 		msg = fmt.Sprintf(i18n.G("Install %q snap from file %q"), snapName, origPath)
 	}
 
-	var userID int
-	if user != nil {
-		userID = user.ID
-	}
-
-	tsets, err := withEnsureCore(st, snapName, userID,
-		func() (*state.TaskSet, error) {
-			return snapstateInstallPath(st, sideInfo, tempPath, "", flags)
-		},
-	)
+	tset, err := snapstateInstallPath(st, sideInfo, tempPath, "", flags)
 	if err != nil {
 		return InternalError("cannot install snap file: %v", err)
 	}
 
-	chg := newChange(st, "install-snap", msg, tsets, []string{snapName})
+	chg := newChange(st, "install-snap", msg, []*state.TaskSet{tset}, []string{snapName})
 	chg.Set("api-data", map[string]string{"snap-name": snapName})
 
 	ensureStateSoon(st)
@@ -1491,7 +1475,7 @@ func iconGet(st *state.State, name string) Response {
 	about, err := localSnapInfo(st, name)
 	if err != nil {
 		if err == errNoSnap {
-			return SnapNotFound(err)
+			return SnapNotFound(name, err)
 		}
 		return InternalError("%v", err)
 	}
@@ -1530,9 +1514,6 @@ func getSnapConf(c *Command, r *http.Request, user *auth.UserState) Response {
 	snapName := vars["name"]
 
 	keys := splitQS(r.URL.Query().Get("keys"))
-	if len(keys) == 0 {
-		return BadRequest("cannot obtain configuration: no keys supplied")
-	}
 
 	s := c.d.overlord.State()
 	s.Lock()
@@ -1540,15 +1521,30 @@ func getSnapConf(c *Command, r *http.Request, user *auth.UserState) Response {
 	s.Unlock()
 
 	currentConfValues := make(map[string]interface{})
+	// Special case - return root document
+	if len(keys) == 0 {
+		keys = []string{""}
+	}
 	for _, key := range keys {
 		var value interface{}
 		if err := tr.Get(snapName, key, &value); err != nil {
 			if config.IsNoOption(err) {
+				if key == "" {
+					// no configuration - return empty document
+					currentConfValues = make(map[string]interface{})
+					break
+				}
 				return BadRequest("%v", err)
 			} else {
 				return InternalError("%v", err)
 			}
 		}
+		if key == "" {
+			if len(keys) > 1 {
+				return BadRequest("keys contains zero-length string")
+			}
+			return SyncResponse(value, nil)
+		}
 
 		currentConfValues[key] = value
 	}
@@ -1561,8 +1557,7 @@ func setSnapConf(c *Command, r *http.Request, user *auth.UserState) Response {
 	snapName := vars["name"]
 
 	var patchValues map[string]interface{}
-	decoder := json.NewDecoder(r.Body)
-	if err := decoder.Decode(&patchValues); err != nil {
+	if err := jsonutil.DecodeWithNumber(r.Body, &patchValues); err != nil {
 		return BadRequest("cannot decode request body into patch values: %v", err)
 	}
 
@@ -1570,17 +1565,14 @@ func setSnapConf(c *Command, r *http.Request, user *auth.UserState) Response {
 	st.Lock()
 	defer st.Unlock()
 
-	var snapst snapstate.SnapState
-	if err := snapstate.Get(st, snapName, &snapst); err != nil {
-		if err == state.ErrNoState {
-			return SnapNotFound(err)
-		} else {
-			return InternalError("%v", err)
+	taskset, err := configstate.ConfigureInstalled(st, snapName, patchValues, 0)
+	if err != nil {
+		if _, ok := err.(*snap.NotInstalledError); ok {
+			return SnapNotFound(snapName, err)
 		}
+		return InternalError("%v", err)
 	}
 
-	taskset := configstate.Configure(st, snapName, patchValues, 0)
-
 	summary := fmt.Sprintf("Change configuration of %q snap", snapName)
 	change := newChange(st, "configure-snap", summary, []*state.TaskSet{taskset}, []string{snapName})
 
@@ -1589,10 +1581,91 @@ func setSnapConf(c *Command, r *http.Request, user *auth.UserState) Response {
 	return AsyncResponse(nil, &Meta{Change: change.ID()})
 }
 
-// getInterfaces returns all plugs and slots.
+// interfacesConnectionsMultiplexer multiplexes to either legacy (connection) or modern behavior (interfaces).
+func interfacesConnectionsMultiplexer(c *Command, r *http.Request, user *auth.UserState) Response {
+	query := r.URL.Query()
+	qselect := query.Get("select")
+	if qselect == "" {
+		return getLegacyConnections(c, r, user)
+	} else {
+		return getInterfaces(c, r, user)
+	}
+}
+
 func getInterfaces(c *Command, r *http.Request, user *auth.UserState) Response {
+	q := r.URL.Query()
+	pselect := q.Get("select")
+	if pselect != "all" && pselect != "connected" {
+		return BadRequest("unsupported select qualifier")
+	}
+	var names []string
+	namesStr := q.Get("names")
+	if namesStr != "" {
+		names = strings.Split(namesStr, ",")
+	}
+
+	opts := &interfaces.InfoOptions{
+		Names:     names,
+		Doc:       q.Get("doc") == "true",
+		Plugs:     q.Get("plugs") == "true",
+		Slots:     q.Get("slots") == "true",
+		Connected: pselect == "connected",
+	}
 	repo := c.d.overlord.InterfaceManager().Repository()
-	return SyncResponse(repo.Interfaces(), nil)
+	return SyncResponse(repo.Info(opts), nil)
+}
+
+func getLegacyConnections(c *Command, r *http.Request, user *auth.UserState) Response {
+	repo := c.d.overlord.InterfaceManager().Repository()
+	ifaces := repo.Interfaces()
+
+	var ifjson interfacesJSON
+
+	plugConns := map[string][]interfaces.SlotRef{}
+	slotConns := map[string][]interfaces.PlugRef{}
+
+	for _, conn := range ifaces.Connections {
+		plugRef := conn.PlugRef.String()
+		slotRef := conn.SlotRef.String()
+		plugConns[plugRef] = append(plugConns[plugRef], conn.SlotRef)
+		slotConns[slotRef] = append(slotConns[slotRef], conn.PlugRef)
+	}
+
+	for _, plug := range ifaces.Plugs {
+		var apps []string
+		for _, app := range plug.Apps {
+			apps = append(apps, app.Name)
+		}
+		pj := plugJSON{
+			Snap:        plug.Snap.Name(),
+			Name:        plug.Name,
+			Interface:   plug.Interface,
+			Attrs:       plug.Attrs,
+			Apps:        apps,
+			Label:       plug.Label,
+			Connections: plugConns[plug.String()],
+		}
+		ifjson.Plugs = append(ifjson.Plugs, pj)
+	}
+	for _, slot := range ifaces.Slots {
+		var apps []string
+		for _, app := range slot.Apps {
+			apps = append(apps, app.Name)
+		}
+
+		sj := slotJSON{
+			Snap:        slot.Snap.Name(),
+			Name:        slot.Name,
+			Interface:   slot.Interface,
+			Attrs:       slot.Attrs,
+			Apps:        apps,
+			Label:       slot.Label,
+			Connections: slotConns[slot.String()],
+		}
+		ifjson.Slots = append(ifjson.Slots, sj)
+	}
+
+	return SyncResponse(ifjson, nil)
 }
 
 // plugJSON aids in marshaling Plug into JSON.
@@ -1617,6 +1690,12 @@ type slotJSON struct {
 	Connections []interfaces.PlugRef   `json:"connections,omitempty"`
 }
 
+// interfacesJSON aids in marshaling plugs, slots and their connections into JSON.
+type interfacesJSON struct {
+	Plugs []plugJSON `json:"plugs,omitempty"`
+	Slots []slotJSON `json:"slots,omitempty"`
+}
+
 // interfaceAction is an action performed on the interface system.
 type interfaceAction struct {
 	Action string     `json:"action"`
@@ -1715,6 +1794,12 @@ func changeInterfaces(c *Command, r *http.Request, user *auth.UserState) Respons
 	return AsyncResponse(nil, &Meta{Change: change.ID()})
 }
 
+func getAssertTypeNames(c *Command, r *http.Request, user *auth.UserState) Response {
+	return SyncResponse(map[string][]string{
+		"types": asserts.TypeNames(),
+	}, nil)
+}
+
 func doAssert(c *Command, r *http.Request, user *auth.UserState) Response {
 	batch := assertstate.NewBatch()
 	_, err := batch.AddStream(r.Body)
@@ -1754,7 +1839,7 @@ func assertsFindMany(c *Command, r *http.Request, user *auth.UserState) Response
 	state.Unlock()
 
 	assertions, err := db.FindMany(assertType, headers)
-	if err == asserts.ErrNotFound {
+	if asserts.IsNotFound(err) {
 		return AssertResponse(nil, true)
 	} else if err != nil {
 		return InternalError("searching assertions failed: %v", err)
@@ -1953,9 +2038,9 @@ func abortChange(c *Command, r *http.Request, user *auth.UserState) Response {
 }
 
 var (
-	postCreateUserUcrednetGetUID = ucrednetGetUID
-	storeUserInfo                = store.UserInfo
-	osutilAddUser                = osutil.AddUser
+	postCreateUserUcrednetGet = ucrednetGet
+	storeUserInfo             = store.UserInfo
+	osutilAddUser             = osutil.AddUser
 )
 
 func getUserDetailsFromStore(email string) (string, *osutil.AddUserOptions, error) {
@@ -1992,7 +2077,7 @@ func createAllKnownSystemUsers(st *state.State, createData *postUserCreateData)
 	st.Lock()
 	assertions, err := db.FindMany(asserts.SystemUserType, headers)
 	st.Unlock()
-	if err != nil && err != asserts.ErrNotFound {
+	if err != nil && !asserts.IsNotFound(err) {
 		return BadRequest("cannot find system-user assertion: %s", err)
 	}
 
@@ -2131,7 +2216,7 @@ func setupLocalUser(st *state.State, username, email string) error {
 }
 
 func postCreateUser(c *Command, r *http.Request, user *auth.UserState) Response {
-	uid, err := postCreateUserUcrednetGetUID(r.RemoteAddr)
+	_, uid, err := postCreateUserUcrednetGet(r.RemoteAddr)
 	if err != nil {
 		return BadRequest("cannot get ucrednet uid: %v", err)
 	}
@@ -2276,6 +2361,8 @@ func postDebug(c *Command, r *http.Request, user *auth.UserState) Response {
 		return SyncResponse(map[string]interface{}{
 			"base-declaration": string(asserts.Encode(bd)),
 		}, nil)
+	case "can-manage-refreshes":
+		return SyncResponse(devicestate.CanManageRefreshes(st), nil)
 	default:
 		return BadRequest("unknown debug action: %v", a.Action)
 	}
@@ -2313,8 +2400,7 @@ func readyToBuy(c *Command, r *http.Request, user *auth.UserState) Response {
 
 func runSnapctl(c *Command, r *http.Request, user *auth.UserState) Response {
 	var snapctlOptions client.SnapCtlOptions
-	decoder := json.NewDecoder(r.Body)
-	if err := decoder.Decode(&snapctlOptions); err != nil {
+	if err := jsonutil.DecodeWithNumber(r.Body, &snapctlOptions); err != nil {
 		return BadRequest("cannot decode snapctl request: %s", err)
 	}
 
@@ -2322,12 +2408,9 @@ func runSnapctl(c *Command, r *http.Request, user *auth.UserState) Response {
 		return BadRequest("snapctl cannot run without args")
 	}
 
-	// Right now snapctl is only used for hooks. If at some point it grows
-	// beyond that, this probably shouldn't go straight to the HookManager.
-	context, err := c.d.overlord.HookManager().Context(snapctlOptions.ContextID)
-	if err != nil {
-		return BadRequest("error running snapctl: %s", err)
-	}
+	// Ignore missing context error to allow 'snapctl -h' without a context;
+	// Actual context is validated later by get/set.
+	context, _ := c.d.overlord.HookManager().Context(snapctlOptions.ContextID)
 	stdout, stderr, err := ctlcmd.Run(context, snapctlOptions.Args)
 	if err != nil {
 		if e, ok := err.(*flags.Error); ok && e.Type == flags.ErrHelp {
@@ -2337,7 +2420,7 @@ func runSnapctl(c *Command, r *http.Request, user *auth.UserState) Response {
 		}
 	}
 
-	if context.IsEphemeral() {
+	if context != nil && context.IsEphemeral() {
 		context.Lock()
 		defer context.Unlock()
 		if err := context.Done(); err != nil {
@@ -2354,7 +2437,7 @@ func runSnapctl(c *Command, r *http.Request, user *auth.UserState) Response {
 }
 
 func getUsers(c *Command, r *http.Request, user *auth.UserState) Response {
-	uid, err := postCreateUserUcrednetGetUID(r.RemoteAddr)
+	_, uid, err := postCreateUserUcrednetGet(r.RemoteAddr)
 	if err != nil {
 		return BadRequest("cannot get ucrednet uid: %v", err)
 	}
@@ -2454,10 +2537,7 @@ func changeAliases(c *Command, r *http.Request, user *auth.UserState) Response {
 		summary = fmt.Sprintf(i18n.G("Prefer aliases of snap %q"), a.Snap)
 	}
 
-	change := st.NewChange(a.Action, summary)
-	change.Set("snap-names", []string{a.Snap})
-	change.AddAll(taskset)
-
+	change := newChange(st, a.Action, summary, []*state.TaskSet{taskset}, []string{a.Snap})
 	st.EnsureBefore(0)
 
 	return AsyncResponse(nil, &Meta{Change: change.ID()})
@@ -2513,3 +2593,110 @@ func getAliases(c *Command, r *http.Request, user *auth.UserState) Response {
 
 	return SyncResponse(res, nil)
 }
+
+func getAppsInfo(c *Command, r *http.Request, user *auth.UserState) Response {
+	query := r.URL.Query()
+
+	opts := appInfoOptions{}
+	switch sel := query.Get("select"); sel {
+	case "":
+		// nothing to do
+	case "service":
+		opts.service = true
+	default:
+		return BadRequest("invalid select parameter: %q", sel)
+	}
+
+	appInfos, rsp := appInfosFor(c.d.overlord.State(), splitQS(query.Get("names")), opts)
+	if rsp != nil {
+		return rsp
+	}
+
+	return SyncResponse(clientAppInfosFromSnapAppInfos(appInfos), nil)
+}
+
+func getLogs(c *Command, r *http.Request, user *auth.UserState) Response {
+	query := r.URL.Query()
+	n := "10"
+	if s := query.Get("n"); s != "" {
+		m, err := strconv.ParseInt(s, 0, 32)
+		if err != nil {
+			return BadRequest(`invalid value for n: %q: %v`, s, err)
+		}
+		if m < 0 {
+			n = "all"
+		} else {
+			n = s
+		}
+	}
+	follow := false
+	if s := query.Get("follow"); s != "" {
+		f, err := strconv.ParseBool(s)
+		if err != nil {
+			return BadRequest(`invalid value for follow: %q: %v`, s, err)
+		}
+		follow = f
+	}
+
+	// only services have logs for now
+	opts := appInfoOptions{service: true}
+	appInfos, rsp := appInfosFor(c.d.overlord.State(), splitQS(query.Get("names")), opts)
+	if rsp != nil {
+		return rsp
+	}
+	if len(appInfos) == 0 {
+		return AppNotFound("no matching services")
+	}
+
+	serviceNames := make([]string, len(appInfos))
+	for i, appInfo := range appInfos {
+		serviceNames[i] = appInfo.ServiceName()
+	}
+
+	sysd := systemd.New(dirs.GlobalRootDir, progress.Null)
+	reader, err := sysd.LogReader(serviceNames, n, follow)
+	if err != nil {
+		return InternalError("cannot get logs: %v", err)
+	}
+
+	return &journalLineReaderSeqResponse{
+		ReadCloser: reader,
+		follow:     follow,
+	}
+}
+
+func postApps(c *Command, r *http.Request, user *auth.UserState) Response {
+	var inst servicestate.Instruction
+	decoder := json.NewDecoder(r.Body)
+	if err := decoder.Decode(&inst); err != nil {
+		return BadRequest("cannot decode request body into service operation: %v", err)
+	}
+	if len(inst.Names) == 0 {
+		// on POST, don't allow empty to mean all
+		return BadRequest("cannot perform operation on services without a list of services to operate on")
+	}
+
+	st := c.d.overlord.State()
+	appInfos, rsp := appInfosFor(st, inst.Names, appInfoOptions{service: true})
+	if rsp != nil {
+		return rsp
+	}
+	if len(appInfos) == 0 {
+		// can't happen: appInfosFor with a non-empty list of services
+		// shouldn't ever return an empty appInfos with no error response
+		return InternalError("no services found")
+	}
+
+	ts, err := servicestate.Control(st, appInfos, &inst, nil)
+	if err != nil {
+		if _, ok := err.(servicestate.ServiceActionConflictError); ok {
+			return Conflict(err.Error())
+		}
+		return BadRequest(err.Error())
+	}
+	st.Lock()
+	defer st.Unlock()
+	chg := newChange(st, "service-control", fmt.Sprintf("Running service command"), []*state.TaskSet{ts}, inst.Names)
+	st.EnsureBefore(0)
+	return AsyncResponse(nil, &Meta{Change: chg.ID()})
+}
