@@ -138,8 +138,7 @@ slots:
 	c.Assert(err, IsNil)
 
 	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, IsNil)
+	c.Assert(slot.Sanitize(s.iface), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestValidSystemBusName(c *C) {
@@ -156,8 +155,7 @@ slots:
 	c.Assert(err, IsNil)
 
 	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, IsNil)
+	c.Assert(slot.Sanitize(s.iface), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestValidFullBusName(c *C) {
@@ -174,8 +172,7 @@ slots:
 	c.Assert(err, IsNil)
 
 	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, IsNil)
+	c.Assert(slot.Sanitize(s.iface), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestNonexistentBusName(c *C) {
@@ -192,9 +189,7 @@ slots:
 	c.Assert(err, IsNil)
 
 	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, Not(IsNil))
-	c.Assert(err, ErrorMatches, "bus 'nonexistent' must be one of 'session' or 'system'")
+	c.Assert(slot.Sanitize(s.iface), ErrorMatches, "bus 'nonexistent' must be one of 'session' or 'system'")
 }
 
 // If this test is failing, be sure to verify the AppArmor rules for binding to
@@ -213,9 +208,7 @@ slots:
 	c.Assert(err, IsNil)
 
 	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, Not(IsNil))
-	c.Assert(err, ErrorMatches, "DBus bus name must not end with -NUMBER")
+	c.Assert(slot.Sanitize(s.iface), ErrorMatches, "DBus bus name must not end with -NUMBER")
 }
 
 func (s *DbusInterfaceSuite) TestSanitizeSlotSystem(c *C) {
@@ -232,8 +225,7 @@ slots:
 	c.Assert(err, IsNil)
 
 	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, IsNil)
+	c.Assert(slot.Sanitize(s.iface), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestSanitizeSlotSession(c *C) {
@@ -250,8 +242,7 @@ slots:
 	c.Assert(err, IsNil)
 
 	slot := &interfaces.Slot{SlotInfo: info.Slots["dbus-slot"]}
-	err = s.iface.SanitizeSlot(slot)
-	c.Assert(err, IsNil)
+	c.Assert(slot.Sanitize(s.iface), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestSanitizePlugSystem(c *C) {
@@ -268,8 +259,7 @@ plugs:
 	c.Assert(err, IsNil)
 
 	plug := &interfaces.Plug{PlugInfo: info.Plugs["dbus-plug"]}
-	err = s.iface.SanitizePlug(plug)
-	c.Assert(err, IsNil)
+	c.Assert(plug.Sanitize(s.iface), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestSanitizePlugSession(c *C) {
@@ -286,13 +276,12 @@ plugs:
 	c.Assert(err, IsNil)
 
 	plug := &interfaces.Plug{PlugInfo: info.Plugs["dbus-plug"]}
-	err = s.iface.SanitizePlug(plug)
-	c.Assert(err, IsNil)
+	c.Assert(plug.Sanitize(s.iface), IsNil)
 }
 
 func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSession(c *C) {
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlot)
+	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlot.SlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
 	snippet := apparmorSpec.SnippetForTag("snap.test-dbus.test-session-provider")
@@ -318,7 +307,7 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSessionNative(c *C) {
 	defer restore()
 
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlot)
+	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlot.SlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
 
@@ -331,7 +320,7 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSessionClassic(c *C) {
 	defer restore()
 
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlot)
+	err := apparmorSpec.AddPermanentSlot(s.iface, s.sessionSlot.SlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-session-provider"})
 
@@ -341,7 +330,7 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSessionClassic(c *C) {
 
 func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSystem(c *C) {
 	apparmorSpec := &apparmor.Specification{}
-	err := apparmorSpec.AddPermanentSlot(s.iface, s.systemSlot)
+	err := apparmorSpec.AddPermanentSlot(s.iface, s.systemSlot.SlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(apparmorSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-system-provider"})
 	snippet := apparmorSpec.SnippetForTag("snap.test-dbus.test-system-provider")
@@ -364,14 +353,14 @@ func (s *DbusInterfaceSuite) TestPermanentSlotAppArmorSystem(c *C) {
 
 func (s *DbusInterfaceSuite) TestPermanentSlotDBusSession(c *C) {
 	dbusSpec := &dbus.Specification{}
-	err := dbusSpec.AddPermanentSlot(s.iface, s.sessionSlot)
+	err := dbusSpec.AddPermanentSlot(s.iface, s.sessionSlot.SlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(dbusSpec.SecurityTags(), HasLen, 0)
 }
 
 func (s *DbusInterfaceSuite) TestPermanentSlotDBusSystem(c *C) {
 	dbusSpec := &dbus.Specification{}
-	err := dbusSpec.AddPermanentSlot(s.iface, s.systemSlot)
+	err := dbusSpec.AddPermanentSlot(s.iface, s.systemSlot.SlotInfo)
 	c.Assert(err, IsNil)
 	c.Assert(dbusSpec.SecurityTags(), DeepEquals, []string{"snap.test-dbus.test-system-provider"})
 	snippet := dbusSpec.SnippetForTag("snap.test-dbus.test-system-provider")
