@@ -28,7 +28,6 @@ import (
 
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/strutil"
-	"github.com/snapcore/snapd/systemd"
 	"github.com/snapcore/snapd/timeout"
 )
 
@@ -726,6 +725,73 @@ slots:
 	})
 }
 
+func (s *YamlSuite) TestUnmarshalGlobalSlotsBindToHooks(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+slots:
+    test-slot:
+hooks:
+    test-hook:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 0)
+	c.Check(info.Slots, HasLen, 1)
+	c.Check(info.Apps, HasLen, 0)
+	c.Check(info.Hooks, HasLen, 1)
+
+	slot, ok := info.Slots["test-slot"]
+	c.Assert(ok, Equals, true, Commentf("Expected slots to include 'test-slot'"))
+	hook, ok := info.Hooks["test-hook"]
+	c.Assert(ok, Equals, true, Commentf("Expected hooks to include 'test-hook'"))
+
+	c.Check(slot, DeepEquals, &snap.SlotInfo{
+		Snap:      info,
+		Name:      "test-slot",
+		Interface: "test-slot",
+		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
+	})
+	c.Check(hook, DeepEquals, &snap.HookInfo{
+		Snap:  info,
+		Name:  "test-hook",
+		Slots: map[string]*snap.SlotInfo{slot.Name: slot},
+	})
+}
+
+func (s *YamlSuite) TestUnmarshalHookWithSlot(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+hooks:
+    test-hook:
+        slots: [test-slot]
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 0)
+	c.Check(info.Slots, HasLen, 1)
+	c.Check(info.Apps, HasLen, 0)
+	c.Check(info.Hooks, HasLen, 1)
+
+	slot, ok := info.Slots["test-slot"]
+	c.Assert(ok, Equals, true, Commentf("Expected slots to include 'test-slot'"))
+	hook, ok := info.Hooks["test-hook"]
+	c.Assert(ok, Equals, true, Commentf("Expected hooks to include 'test-hook'"))
+
+	c.Check(slot, DeepEquals, &snap.SlotInfo{
+		Snap:      info,
+		Name:      "test-slot",
+		Interface: "test-slot",
+		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
+	})
+	c.Check(hook, DeepEquals, &snap.HookInfo{
+		Snap:  info,
+		Name:  "test-hook",
+		Slots: map[string]*snap.SlotInfo{slot.Name: slot},
+	})
+}
+
 func (s *YamlSuite) TestUnmarshalCorruptedSlotWithNonStringInterfaceName(c *C) {
 	// NOTE: yaml content cannot use tabs, indent the section with spaces.
 	_, err := snap.InfoFromSnapYaml([]byte(`
@@ -1058,6 +1124,7 @@ summary: foo app
 type: app
 epoch: 1*
 confinement: devmode
+license: GPL-3.0
 description: |
     Foo provides useful services
 apps:
@@ -1071,6 +1138,7 @@ apps:
 hooks:
     test-hook:
        plugs: [foo-socket-plug]
+       slots: [foo-socket-slot]
 plugs:
     foo-socket-plug:
         interface: socket
@@ -1089,7 +1157,7 @@ slots:
 	c.Check(info.Name(), Equals, "foo")
 	c.Check(info.Version, Equals, "1.2")
 	c.Check(info.Type, Equals, snap.TypeApp)
-	c.Check(info.Epoch, Equals, "1*")
+	c.Check(info.Epoch.String(), Equals, "1*")
 	c.Check(info.Confinement, Equals, snap.DevModeConfinement)
 	c.Check(info.Title(), Equals, "Foo")
 	c.Check(info.Summary(), Equals, "foo app")
@@ -1101,6 +1169,7 @@ slots:
 	c.Check(info.Publisher, Equals, "")
 	c.Check(info.PublisherID, Equals, "")
 	c.Check(info.Channel, Equals, "")
+	c.Check(info.License, Equals, "GPL-3.0")
 
 	app1 := info.Apps["daemon"]
 	app2 := info.Apps["foo"]
@@ -1138,14 +1207,16 @@ slots:
 	c.Check(app2.Slots, DeepEquals, map[string]*snap.SlotInfo{
 		slot2.Name: slot2})
 
-	// hook1 has two plugs ("foo-socket", "logging"). The plug "logging" is
-	// global while "foo-socket" is hook-bound.
+	// hook1 has two plugs ("foo-socket", "logging") and two slots ("foo-socket", "tracing").
+	// The plug "logging" and slot "tracing" are global while "foo-socket" is hook-bound.
 
 	c.Assert(hook, NotNil)
 	c.Check(hook.Snap, Equals, info)
 	c.Check(hook.Name, Equals, "test-hook")
 	c.Check(hook.Plugs, DeepEquals, map[string]*snap.PlugInfo{
 		plug3.Name: plug3, plug4.Name: plug4})
+	c.Check(hook.Slots, DeepEquals, map[string]*snap.SlotInfo{
+		slot1.Name: slot1, slot2.Name: slot2})
 
 	// plug1 ("network") is implicitly defined and app-bound to "daemon"
 
@@ -1228,7 +1299,7 @@ version: 1.0
 `)
 	info, err := snap.InfoFromSnapYaml(y)
 	c.Assert(err, IsNil)
-	c.Assert(info.Epoch, Equals, "0")
+	c.Assert(info.Epoch.String(), Equals, "0")
 }
 
 func (s *YamlSuite) TestSnapYamlConfinementDefault(c *C) {
@@ -1335,24 +1406,84 @@ apps:
    post-stop-command: post-stop-cmd
    restart-condition: on-abnormal
    bus-name: busName
+   sockets:
+     sock1:
+       listen-stream: $SNAP_DATA/sock1.socket
+       socket-mode: 0666
 `)
 	info, err := snap.InfoFromSnapYaml(y)
 	c.Assert(err, IsNil)
+
+	app := snap.AppInfo{
+		Snap:            info,
+		Name:            "svc",
+		Command:         "svc1",
+		Daemon:          "forking",
+		RestartCond:     snap.RestartOnAbnormal,
+		StopTimeout:     timeout.Timeout(25 * time.Second),
+		StopCommand:     "stop-cmd",
+		PostStopCommand: "post-stop-cmd",
+		BusName:         "busName",
+		Sockets:         map[string]*snap.SocketInfo{},
+	}
+
+	app.Sockets["sock1"] = &snap.SocketInfo{
+		App:          &app,
+		Name:         "sock1",
+		ListenStream: "$SNAP_DATA/sock1.socket",
+		SocketMode:   0666,
+	}
+
+	c.Check(info.Apps, DeepEquals, map[string]*snap.AppInfo{"svc": &app})
+}
+
+func (s *YamlSuite) TestDaemonListenStreamAsInteger(c *C) {
+	y := []byte(`name: wat
+version: 42
+apps:
+ svc:
+   command: svc
+   sockets:
+     sock:
+       listen-stream: 8080
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+
+	app := snap.AppInfo{
+		Snap:    info,
+		Name:    "svc",
+		Command: "svc",
+		Sockets: map[string]*snap.SocketInfo{},
+	}
+
+	app.Sockets["sock"] = &snap.SocketInfo{
+		App:          &app,
+		Name:         "sock",
+		ListenStream: "8080",
+	}
+
 	c.Check(info.Apps, DeepEquals, map[string]*snap.AppInfo{
-		"svc": {
-			Snap:            info,
-			Name:            "svc",
-			Command:         "svc1",
-			Daemon:          "forking",
-			RestartCond:     systemd.RestartOnAbnormal,
-			StopTimeout:     timeout.Timeout(25 * time.Second),
-			StopCommand:     "stop-cmd",
-			PostStopCommand: "post-stop-cmd",
-			BusName:         "busName",
-		},
+		"svc": &app,
 	})
 }
 
+func (s *YamlSuite) TestDaemonInvalidSocketMode(c *C) {
+	y := []byte(`name: wat
+version: 42
+apps:
+ svc:
+   command: svc
+   sockets:
+     sock:
+       listen-stream: 8080
+       socket-mode: asdfasdf
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Check(err.Error(), Equals, "info failed to parse: yaml: unmarshal errors:\n"+
+		"  line 9: cannot unmarshal !!str `asdfasdf` into os.FileMode")
+}
+
 func (s *YamlSuite) TestSnapYamlGlobalEnvironment(c *C) {
 	y := []byte(`
 name: foo
