@@ -69,11 +69,17 @@ const (
 	UbuntuCoreWireProtocol = "1"
 )
 
+type RefreshOptions struct {
+	// RefreshManaged indicates to the store that the refresh is
+	// managed via snapd-control.
+	RefreshManaged bool
+}
+
 // the LimitTime should be slightly more than 3 times of our http.Client
 // Timeout value
-var defaultRetryStrategy = retry.LimitCount(5, retry.LimitTime(33*time.Second,
+var defaultRetryStrategy = retry.LimitCount(5, retry.LimitTime(38*time.Second,
 	retry.Exponential{
-		Initial: 100 * time.Millisecond,
+		Initial: 300 * time.Millisecond,
 		Factor:  2.5,
 	},
 ))
@@ -83,7 +89,7 @@ func infoFromRemote(d *snapDetails) *snap.Info {
 	info.Architectures = d.Architectures
 	info.Type = d.Type
 	info.Version = d.Version
-	info.Epoch = "0"
+	info.Epoch = d.Epoch
 	info.RealName = d.Name
 	info.SnapID = d.SnapID
 	info.Revision = snap.R(d.Revision)
@@ -100,8 +106,11 @@ func infoFromRemote(d *snapDetails) *snap.Info {
 	info.DownloadURL = d.DownloadURL
 	info.Prices = d.Prices
 	info.Private = d.Private
+	info.Paid = len(info.Prices) > 0
 	info.Confinement = snap.ConfinementType(d.Confinement)
 	info.Contact = d.Contact
+	info.License = d.License
+	info.Base = d.Base
 
 	deltas := make([]snap.DeltaInfo, len(d.Deltas))
 	for i, d := range d.Deltas {
@@ -164,14 +173,10 @@ func infoFromRemote(d *snapDetails) *snap.Info {
 
 // Config represents the configuration to access the snap store
 type Config struct {
-	SearchURI      *url.URL
-	DetailsURI     *url.URL
-	BulkURI        *url.URL
-	AssertionsURI  *url.URL
-	OrdersURI      *url.URL
-	BuyURI         *url.URL
-	CustomersMeURI *url.URL
-	SectionsURI    *url.URL
+	// Store API base URLs. The assertions url is only separate because it can
+	// be overridden by its own env var.
+	StoreBaseURL      *url.URL
+	AssertionsBaseURL *url.URL
 
 	// StoreID is the store id used if we can't get one through the AuthContext.
 	StoreID string
@@ -181,18 +186,33 @@ type Config struct {
 
 	DetailFields []string
 	DeltaFormat  string
+
+	// CacheDownloads is the number of downloads that should be cached
+	CacheDownloads int
+}
+
+// setBaseURL updates the store API's base URL in the Config. Must not be used
+// to change active config.
+func (cfg *Config) setBaseURL(u *url.URL) error {
+	storeBaseURI, err := storeURL(u)
+	if err != nil {
+		return err
+	}
+
+	assertsBaseURI, err := assertsURL()
+	if err != nil {
+		return err
+	}
+
+	cfg.StoreBaseURL = storeBaseURI
+	cfg.AssertionsBaseURL = assertsBaseURI
+
+	return nil
 }
 
 // Store represents the ubuntu snap store
 type Store struct {
-	searchURI      *url.URL
-	detailsURI     *url.URL
-	bulkURI        *url.URL
-	assertionsURI  *url.URL
-	ordersURI      *url.URL
-	buyURI         *url.URL
-	customersMeURI *url.URL
-	sectionsURI    *url.URL
+	cfg *Config
 
 	architecture string
 	series       string
@@ -210,6 +230,8 @@ type Store struct {
 
 	mu                sync.Mutex
 	suggestedCurrency string
+
+	cacher downloadCache
 }
 
 func respToError(resp *http.Response, msg string) error {
@@ -256,36 +278,65 @@ func useStaging() bool {
 	return osutil.GetenvBool("SNAPPY_USE_STAGING_STORE")
 }
 
-// Extend a base URL with additional unescaped paths.  (url.Parse handles
-// resolving relative links, which isn't quite what we want: that goes wrong if
-// the base URL doesn't end with a slash.)
-func urlJoin(base *url.URL, paths ...string) *url.URL {
-	if len(paths) == 0 {
-		return base
+// endpointURL clones a base URL and updates it with optional path and query.
+func endpointURL(base *url.URL, path string, query url.Values) *url.URL {
+	u := *base
+	if path != "" {
+		u.Path = strings.TrimSuffix(u.Path, "/") + "/" + strings.TrimPrefix(path, "/")
+		u.RawQuery = ""
 	}
-	url := *base
-	url.RawQuery = ""
-	paths = append([]string{strings.TrimSuffix(url.Path, "/")}, paths...)
-	url.Path = strings.Join(paths, "/")
-	return &url
+	if len(query) != 0 {
+		u.RawQuery = query.Encode()
+	}
+	return &u
 }
 
-func apiURL() string {
-	// FIXME: this will become a store-url assertion
+// apiURL returns the system default base API URL.
+func apiURL() *url.URL {
+	s := "https://api.snapcraft.io/"
+	if useStaging() {
+		s = "https://api.staging.snapcraft.io/"
+	}
+	u, _ := url.Parse(s)
+	return u
+}
+
+// storeURL returns the base store URL, derived from either the given API URL
+// or an env var override.
+func storeURL(api *url.URL) (*url.URL, error) {
+	var override string
+	var overrideName string
 	// XXX: Deprecated but present for backward-compatibility: this used
 	// to be "Click Package Index".  Remove this once people have got
 	// used to SNAPPY_FORCE_API_URL instead.
-	if u := os.Getenv("SNAPPY_FORCE_CPI_URL"); u != "" && strings.HasSuffix(u, "api/v1/") {
-		return strings.TrimSuffix(u, "api/v1/")
+	if s := os.Getenv("SNAPPY_FORCE_CPI_URL"); s != "" && strings.HasSuffix(s, "api/v1/") {
+		overrideName = "SNAPPY_FORCE_CPI_URL"
+		override = strings.TrimSuffix(s, "api/v1/")
+	} else if s := os.Getenv("SNAPPY_FORCE_API_URL"); s != "" {
+		overrideName = "SNAPPY_FORCE_API_URL"
+		override = s
 	}
-	if u := os.Getenv("SNAPPY_FORCE_API_URL"); u != "" {
-		return u
+	if override != "" {
+		u, err := url.Parse(override)
+		if err != nil {
+			return nil, fmt.Errorf("invalid %s: %s", overrideName, err)
+		}
+		return u, nil
 	}
-	if useStaging() {
-		return "https://api.staging.snapcraft.io/"
+	return api, nil
+}
+
+func assertsURL() (*url.URL, error) {
+	if s := os.Getenv("SNAPPY_FORCE_SAS_URL"); s != "" {
+		u, err := url.Parse(s)
+		if err != nil {
+			return nil, fmt.Errorf("invalid SNAPPY_FORCE_SAS_URL: %s", err)
+		}
+		return u, nil
 	}
 
-	return "https://api.snapcraft.io/"
+	// nil means fallback to store base url
+	return nil, nil
 }
 
 func authLocation() string {
@@ -302,15 +353,6 @@ func authURL() string {
 	return "https://" + authLocation() + "/api/v2"
 }
 
-func assertsURL(storeBaseURI *url.URL) string {
-	if u := os.Getenv("SNAPPY_FORCE_SAS_URL"); u != "" {
-		return u
-	}
-	// XXX: This will eventually become urlJoin(storeBaseURI, "v2/")
-	// once new bulk-friendly APIs are designed and implemented.
-	return urlJoin(storeBaseURI, "api/v1/snaps/").String()
-}
-
 func myappsURL() string {
 	if useStaging() {
 		return "https://myapps.developer.staging.ubuntu.com/"
@@ -327,34 +369,17 @@ func DefaultConfig() *Config {
 }
 
 func init() {
-	storeBaseURI, err := url.Parse(apiURL())
+	storeBaseURI, err := storeURL(apiURL())
 	if err != nil {
 		panic(err)
 	}
 	if storeBaseURI.RawQuery != "" {
 		panic("store API URL may not contain query string")
 	}
-
-	assertsBaseURI, err := url.Parse(assertsURL(storeBaseURI))
+	err = defaultConfig.setBaseURL(storeBaseURI)
 	if err != nil {
 		panic(err)
 	}
-
-	// XXX: Repeating "api/" here is cumbersome, but the next generation
-	// of store APIs will probably drop that prefix (since it now
-	// duplicates the hostname), and we may want to switch to v2 APIs
-	// one at a time; so it's better to consider that as part of
-	// individual endpoint paths.
-	defaultConfig.SearchURI = urlJoin(storeBaseURI, "api/v1/snaps/search")
-	// slash at the end because snap name is appended to this with .Parse(snapName)
-	defaultConfig.DetailsURI = urlJoin(storeBaseURI, "api/v1/snaps/details/")
-	defaultConfig.BulkURI = urlJoin(storeBaseURI, "api/v1/snaps/metadata")
-	defaultConfig.OrdersURI = urlJoin(storeBaseURI, "api/v1/snaps/purchases/orders")
-	defaultConfig.BuyURI = urlJoin(storeBaseURI, "api/v1/snaps/purchases/buy")
-	defaultConfig.CustomersMeURI = urlJoin(storeBaseURI, "api/v1/snaps/purchases/customers/me")
-	defaultConfig.SectionsURI = urlJoin(storeBaseURI, "api/v1/snaps/sections")
-
-	defaultConfig.AssertionsURI = urlJoin(assertsBaseURI, "assertions/")
 }
 
 type searchResults struct {
@@ -389,32 +414,6 @@ func New(cfg *Config, authContext auth.AuthContext) *Store {
 		fields = detailFields
 	}
 
-	rawQuery := ""
-	if len(fields) > 0 {
-		v := url.Values{}
-		v.Set("fields", strings.Join(fields, ","))
-		rawQuery = v.Encode()
-	}
-
-	var searchURI *url.URL
-	if cfg.SearchURI != nil {
-		uri := *cfg.SearchURI
-		uri.RawQuery = rawQuery
-		searchURI = &uri
-	}
-
-	var detailsURI *url.URL
-	if cfg.DetailsURI != nil {
-		uri := *cfg.DetailsURI
-		uri.RawQuery = rawQuery
-		detailsURI = &uri
-	}
-
-	var sectionsURI *url.URL
-	if cfg.SectionsURI != nil {
-		sectionsURI = cfg.SectionsURI
-	}
-
 	architecture := arch.UbuntuArchitecture()
 	if cfg.Architecture != "" {
 		architecture = cfg.Architecture
@@ -430,16 +429,8 @@ func New(cfg *Config, authContext auth.AuthContext) *Store {
 		deltaFormat = defaultSupportedDeltaFormat
 	}
 
-	// see https://wiki.ubuntu.com/AppStore/Interfaces/ClickPackageIndex
-	return &Store{
-		searchURI:       searchURI,
-		detailsURI:      detailsURI,
-		bulkURI:         cfg.BulkURI,
-		assertionsURI:   cfg.AssertionsURI,
-		ordersURI:       cfg.OrdersURI,
-		buyURI:          cfg.BuyURI,
-		customersMeURI:  cfg.CustomersMeURI,
-		sectionsURI:     sectionsURI,
+	store := &Store{
+		cfg:             cfg,
 		series:          series,
 		architecture:    architecture,
 		noCDN:           osutil.GetenvBool("SNAPPY_STORE_NO_CDN"),
@@ -453,6 +444,68 @@ func New(cfg *Config, authContext auth.AuthContext) *Store {
 			MayLogBody: true,
 		}),
 	}
+	store.SetCacheDownloads(cfg.CacheDownloads)
+
+	return store
+}
+
+// API endpoint paths
+const (
+	// see https://wiki.ubuntu.com/AppStore/Interfaces/ClickPackageIndex
+	// XXX: Repeating "api/" here is cumbersome, but the next generation
+	// of store APIs will probably drop that prefix (since it now
+	// duplicates the hostname), and we may want to switch to v2 APIs
+	// one at a time; so it's better to consider that as part of
+	// individual endpoint paths.
+	searchEndpPath      = "api/v1/snaps/search"
+	detailsEndpPath     = "api/v1/snaps/details"
+	bulkEndpPath        = "api/v1/snaps/metadata"
+	ordersEndpPath      = "api/v1/snaps/purchases/orders"
+	buyEndpPath         = "api/v1/snaps/purchases/buy"
+	customersMeEndpPath = "api/v1/snaps/purchases/customers/me"
+	sectionsEndpPath    = "api/v1/snaps/sections"
+	commandsEndpPath    = "api/v1/snaps/names"
+
+	deviceNonceEndpPath   = "api/v1/snaps/auth/nonces"
+	deviceSessionEndpPath = "api/v1/snaps/auth/sessions"
+
+	assertionsPath = "api/v1/snaps/assertions"
+)
+
+func (s *Store) defaultSnapQuery() url.Values {
+	q := url.Values{}
+	if len(s.detailFields) != 0 {
+		q.Set("fields", strings.Join(s.detailFields, ","))
+	}
+	return q
+}
+
+func (s *Store) baseURL(defaultURL *url.URL) *url.URL {
+	u := defaultURL
+	if s.authContext != nil {
+		var err error
+		_, u, err = s.authContext.ProxyStoreParams(defaultURL)
+		if err != nil {
+			logger.Debugf("cannot get proxy store parameters from state: %v", err)
+		}
+	}
+	if u != nil {
+		return u
+	}
+	return defaultURL
+}
+
+func (s *Store) endpointURL(p string, query url.Values) *url.URL {
+	return endpointURL(s.baseURL(s.cfg.StoreBaseURL), p, query)
+}
+
+func (s *Store) assertionsEndpointURL(p string, query url.Values) *url.URL {
+	defBaseURL := s.cfg.StoreBaseURL
+	// can be overridden separately!
+	if s.cfg.AssertionsBaseURL != nil {
+		defBaseURL = s.cfg.AssertionsBaseURL
+	}
+	return endpointURL(s.baseURL(defBaseURL), path.Join(assertionsPath, p), query)
 }
 
 // LoginUser logs user in the store and returns the authentication macaroons.
@@ -581,17 +634,17 @@ func (s *Store) refreshDeviceSession(device *auth.DeviceState) error {
 		return fmt.Errorf("internal error: no authContext")
 	}
 
-	nonce, err := requestStoreDeviceNonce()
+	nonce, err := requestStoreDeviceNonce(s.endpointURL(deviceNonceEndpPath, nil).String())
 	if err != nil {
 		return err
 	}
 
-	sessionRequest, serialAssertion, err := s.authContext.DeviceSessionRequest(nonce)
+	devSessReqParams, err := s.authContext.DeviceSessionRequestParams(nonce)
 	if err != nil {
 		return err
 	}
 
-	session, err := requestDeviceSession(string(serialAssertion), string(sessionRequest), device.SessionMacaroon)
+	session, err := requestDeviceSession(s.endpointURL(deviceSessionEndpPath, nil).String(), devSessReqParams, device.SessionMacaroon)
 	if err != nil {
 		return err
 	}
@@ -637,6 +690,13 @@ type requestOptions struct {
 	Data         []byte
 }
 
+func (r *requestOptions) addHeader(k, v string) {
+	if r.ExtraHeaders == nil {
+		r.ExtraHeaders = make(map[string]string)
+	}
+	r.ExtraHeaders[k] = v
+}
+
 func cancelled(ctx context.Context) bool {
 	select {
 	case <-ctx.Done():
@@ -646,6 +706,47 @@ func cancelled(ctx context.Context) bool {
 	}
 }
 
+var expectedCatalogPreamble = []interface{}{
+	json.Delim('{'),
+	"_embedded",
+	json.Delim('{'),
+	"clickindex:package",
+	json.Delim('['),
+}
+
+type catalogItem struct {
+	Name string `json:"package_name"`
+}
+
+func decodeCatalog(resp *http.Response, names io.Writer) error {
+	const what = "decode new commands catalog"
+	if resp.StatusCode != 200 {
+		return respToError(resp, what)
+	}
+	dec := json.NewDecoder(resp.Body)
+	for _, expectedToken := range expectedCatalogPreamble {
+		token, err := dec.Token()
+		if err != nil {
+			return err
+		}
+		if token != expectedToken {
+			return fmt.Errorf(what+": bad catalog preamble: expected %#v, got %#v", expectedToken, token)
+		}
+	}
+
+	for dec.More() {
+		var v catalogItem
+		if err := dec.Decode(&v); err != nil {
+			return fmt.Errorf(what+": %v", err)
+		}
+		if v.Name != "" {
+			fmt.Fprintln(names, v.Name)
+		}
+	}
+
+	return nil
+}
+
 func decodeJSONBody(resp *http.Response, success interface{}, failure interface{}) error {
 	ok := (resp.StatusCode == 200 || resp.StatusCode == 201)
 	// always decode on success; decode failures only if body is not empty
@@ -834,7 +935,7 @@ func (s *Store) decorateOrders(snaps []*snap.Info, user *auth.UserState) error {
 	// Mark every non-free snap as must buy until we know better.
 	hasPriced := false
 	for _, info := range snaps {
-		if len(info.Prices) != 0 {
+		if info.Paid {
 			info.MustBuy = true
 			hasPriced = true
 		}
@@ -852,7 +953,7 @@ func (s *Store) decorateOrders(snaps []*snap.Info, user *auth.UserState) error {
 
 	reqOptions := &requestOptions{
 		Method: "GET",
-		URL:    s.ordersURI,
+		URL:    s.endpointURL(ordersEndpPath, nil),
 		Accept: jsonContentType,
 	}
 	var result ordersResult
@@ -876,15 +977,15 @@ func (s *Store) decorateOrders(snaps []*snap.Info, user *auth.UserState) error {
 	}
 
 	for _, info := range snaps {
-		info.MustBuy = mustBuy(info.Prices, bought[info.SnapID])
+		info.MustBuy = mustBuy(info.Paid, bought[info.SnapID])
 	}
 
 	return nil
 }
 
 // mustBuy determines if a snap requires a payment, based on if it is non-free and if the user has already bought it
-func mustBuy(prices map[string]float64, bought bool) bool {
-	if len(prices) == 0 {
+func mustBuy(paid bool, bought bool) bool {
+	if !paid {
 		// If the snap is free, then it doesn't need buying
 		return false
 	}
@@ -904,12 +1005,7 @@ type SnapSpec struct {
 
 // SnapInfo returns the snap.Info for the store-hosted snap matching the given spec, or an error.
 func (s *Store) SnapInfo(snapSpec SnapSpec, user *auth.UserState) (*snap.Info, error) {
-	// get the query before doing Parse, as that overwrites it
-	query := s.detailsURI.Query()
-	u, err := s.detailsURI.Parse(snapSpec.Name)
-	if err != nil {
-		return nil, err
-	}
+	query := s.defaultSnapQuery()
 
 	channel := snapSpec.Channel
 	var sel string
@@ -929,8 +1025,7 @@ func (s *Store) SnapInfo(snapSpec SnapSpec, user *auth.UserState) (*snap.Info, e
 	}
 	query.Set("channel", channel)
 
-	u.RawQuery = query.Encode()
-
+	u := s.endpointURL(path.Join(detailsEndpPath, snapSpec.Name), query)
 	reqOptions := &requestOptions{
 		Method: "GET",
 		URL:    u,
@@ -995,8 +1090,7 @@ func (s *Store) Find(search *Search, user *auth.UserState) ([]*snap.Info, error)
 		return nil, ErrBadQuery
 	}
 
-	u := *s.searchURI // make a copy, so we can mutate it
-	q := u.Query()
+	q := s.defaultSnapQuery()
 
 	if search.Private {
 		if search.Prefix {
@@ -1022,11 +1116,11 @@ func (s *Store) Find(search *Search, user *auth.UserState) ([]*snap.Info, error)
 	} else {
 		q.Set("confinement", "strict")
 	}
-	u.RawQuery = q.Encode()
 
+	u := s.endpointURL(searchEndpPath, q)
 	reqOptions := &requestOptions{
 		Method: "GET",
-		URL:    &u,
+		URL:    u,
 		Accept: halJsonContentType,
 	}
 
@@ -1061,15 +1155,9 @@ func (s *Store) Find(search *Search, user *auth.UserState) ([]*snap.Info, error)
 
 // Sections retrieves the list of available store sections.
 func (s *Store) Sections(user *auth.UserState) ([]string, error) {
-	u := *s.sectionsURI // make a copy, so we can mutate it
-
-	q := u.Query()
-
-	u.RawQuery = q.Encode()
-
 	reqOptions := &requestOptions{
 		Method: "GET",
-		URL:    &u,
+		URL:    s.endpointURL(sectionsEndpPath, nil),
 		Accept: halJsonContentType,
 	}
 
@@ -1095,25 +1183,70 @@ func (s *Store) Sections(user *auth.UserState) ([]string, error) {
 	return sectionNames, nil
 }
 
+// WriteCatalogs queries the "commands" endpoint and writes the
+// command names into the given io.Writer.
+func (s *Store) WriteCatalogs(names io.Writer) error {
+	u := *s.endpointURL(commandsEndpPath, nil)
+
+	q := u.Query()
+	if release.OnClassic {
+		q.Set("confinement", "strict,classic")
+	} else {
+		q.Set("confinement", "strict")
+	}
+
+	u.RawQuery = q.Encode()
+	reqOptions := &requestOptions{
+		Method: "GET",
+		URL:    &u,
+		Accept: halJsonContentType,
+	}
+
+	// do not log body for catalog updates (its huge)
+	client := httputil.NewHTTPClient(&httputil.ClientOpts{
+		MayLogBody: false,
+		Timeout:    10 * time.Second,
+	})
+	doRequest := func() (*http.Response, error) {
+		return s.doRequest(context.TODO(), client, reqOptions, nil)
+	}
+	readResponse := func(resp *http.Response) error {
+		return decodeCatalog(resp, names)
+	}
+
+	resp, err := httputil.RetryRequest(u.String(), doRequest, readResponse, defaultRetryStrategy)
+	if err != nil {
+		return err
+	}
+	if resp.StatusCode != 200 {
+		return respToError(resp, "refresh commands catalog")
+	}
+
+	return nil
+}
+
 // RefreshCandidate contains information for the store about the currently
 // installed snap so that the store can decide what update we should see
 type RefreshCandidate struct {
 	SnapID   string
 	Revision snap.Revision
-	Epoch    string
+	Epoch    snap.Epoch
 	Block    []snap.Revision
 
 	// the desired channel
 	Channel string
+	// whether validation should be ignored
+	IgnoreValidation bool
 }
 
 // the exact bits that we need to send to the store
 type currentSnapJSON struct {
-	SnapID      string `json:"snap_id"`
-	Channel     string `json:"channel"`
-	Revision    int    `json:"revision,omitempty"`
-	Epoch       string `json:"epoch"`
-	Confinement string `json:"confinement"`
+	SnapID           string     `json:"snap_id"`
+	Channel          string     `json:"channel"`
+	Revision         int        `json:"revision,omitempty"`
+	Epoch            snap.Epoch `json:"epoch"`
+	Confinement      string     `json:"confinement"`
+	IgnoreValidation bool       `json:"ignore_validation,omitempty"`
 }
 
 type metadataWrapper struct {
@@ -1141,16 +1274,21 @@ func currentSnap(cs *RefreshCandidate) *currentSnapJSON {
 	}
 
 	return &currentSnapJSON{
-		SnapID:   cs.SnapID,
-		Channel:  channel,
-		Epoch:    cs.Epoch,
-		Revision: cs.Revision.N,
+		SnapID:           cs.SnapID,
+		Channel:          channel,
+		Epoch:            cs.Epoch,
+		Revision:         cs.Revision.N,
+		IgnoreValidation: cs.IgnoreValidation,
 		// confinement purposely left empty
 	}
 }
 
 // query the store for the information about currently offered revisions of snaps
-func (s *Store) refreshForCandidates(currentSnaps []*currentSnapJSON, user *auth.UserState) ([]*snapDetails, error) {
+func (s *Store) refreshForCandidates(currentSnaps []*currentSnapJSON, user *auth.UserState, flags *RefreshOptions) ([]*snapDetails, error) {
+	if flags == nil {
+		flags = &RefreshOptions{}
+	}
+
 	if len(currentSnaps) == 0 {
 		// nothing to do
 		return nil, nil
@@ -1167,7 +1305,7 @@ func (s *Store) refreshForCandidates(currentSnaps []*currentSnapJSON, user *auth
 
 	reqOptions := &requestOptions{
 		Method:      "POST",
-		URL:         s.bulkURI,
+		URL:         s.endpointURL(bulkEndpPath, nil),
 		Accept:      halJsonContentType,
 		ContentType: jsonContentType,
 		Data:        jsonData,
@@ -1175,9 +1313,10 @@ func (s *Store) refreshForCandidates(currentSnaps []*currentSnapJSON, user *auth
 
 	if useDeltas() {
 		logger.Debugf("Deltas enabled. Adding header X-Ubuntu-Delta-Formats: %v", s.deltaFormat)
-		reqOptions.ExtraHeaders = map[string]string{
-			"X-Ubuntu-Delta-Formats": s.deltaFormat,
-		}
+		reqOptions.addHeader("X-Ubuntu-Delta-Formats", s.deltaFormat)
+	}
+	if flags.RefreshManaged {
+		reqOptions.addHeader("X-Ubuntu-Refresh-Managed", "true")
 	}
 
 	var updateData searchResults
@@ -1204,7 +1343,7 @@ func (s *Store) LookupRefresh(installed *RefreshCandidate, user *auth.UserState)
 		return nil, ErrLocalSnap
 	}
 
-	latest, err := refreshForCandidates(s, []*currentSnapJSON{cur}, user)
+	latest, err := refreshForCandidates(s, []*currentSnapJSON{cur}, user, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -1223,8 +1362,7 @@ func (s *Store) LookupRefresh(installed *RefreshCandidate, user *auth.UserState)
 
 // ListRefresh returns the available updates for a list of refresh candidates.
 // NOTE ListRefresh can return nil, nil if e.g. all local snaps are passed in
-func (s *Store) ListRefresh(installed []*RefreshCandidate, user *auth.UserState) (snaps []*snap.Info, err error) {
-
+func (s *Store) ListRefresh(installed []*RefreshCandidate, user *auth.UserState, flags *RefreshOptions) (snaps []*snap.Info, err error) {
 	candidateMap := map[string]*RefreshCandidate{}
 	currentSnaps := make([]*currentSnapJSON, 0, len(installed))
 	for _, cs := range installed {
@@ -1236,7 +1374,7 @@ func (s *Store) ListRefresh(installed []*RefreshCandidate, user *auth.UserState)
 		candidateMap[cs.SnapID] = cs
 	}
 
-	latest, err := s.refreshForCandidates(currentSnaps, user)
+	latest, err := s.refreshForCandidates(currentSnaps, user, flags)
 	if err != nil {
 		return nil, err
 	}
@@ -1285,16 +1423,22 @@ func (s *Store) Download(ctx context.Context, name string, targetPath string, do
 	if err := os.MkdirAll(filepath.Dir(targetPath), 0755); err != nil {
 		return err
 	}
+
+	if err := s.cacher.Get(downloadInfo.Sha3_384, targetPath); err == nil {
+		return nil
+	}
+
 	if useDeltas() {
 		logger.Debugf("Available deltas returned by store: %v", downloadInfo.Deltas)
-	}
-	if useDeltas() && len(downloadInfo.Deltas) == 1 {
-		err := s.downloadAndApplyDelta(name, targetPath, downloadInfo, pbar, user)
-		if err == nil {
-			return nil
+
+		if len(downloadInfo.Deltas) == 1 {
+			err := s.downloadAndApplyDelta(name, targetPath, downloadInfo, pbar, user)
+			if err == nil {
+				return nil
+			}
+			// We revert to normal downloads if there is any error.
+			logger.Noticef("Cannot download or apply deltas for %s: %v", name, err)
 		}
-		// We revert to normal downloads if there is any error.
-		logger.Noticef("Cannot download or apply deltas for %s: %v", name, err)
 	}
 
 	partialPath := targetPath + ".partial"
@@ -1325,7 +1469,22 @@ func (s *Store) Download(ctx context.Context, name string, targetPath string, do
 		url = downloadInfo.DownloadURL
 	}
 
-	err = download(ctx, name, downloadInfo.Sha3_384, url, user, s, w, resume, pbar)
+	if downloadInfo.Size == 0 || resume < downloadInfo.Size {
+		err = download(ctx, name, downloadInfo.Sha3_384, url, user, s, w, resume, pbar)
+	} else {
+		// we're done! check the hash though
+		h := crypto.SHA3_384.New()
+		if _, err := w.Seek(0, os.SEEK_SET); err != nil {
+			return err
+		}
+		if _, err := io.Copy(h, w); err != nil {
+			return err
+		}
+		actualSha3 := fmt.Sprintf("%x", h.Sum(nil))
+		if downloadInfo.Sha3_384 != actualSha3 {
+			err = HashError{name, actualSha3, downloadInfo.Sha3_384}
+		}
+	}
 	// If hashsum is incorrect retry once
 	if _, ok := err.(HashError); ok {
 		logger.Debugf("Hashsum error on download: %v", err.Error())
@@ -1348,7 +1507,11 @@ func (s *Store) Download(ctx context.Context, name string, targetPath string, do
 		return err
 	}
 
-	return w.Sync()
+	if err := w.Sync(); err != nil {
+		return err
+	}
+
+	return s.cacher.Put(downloadInfo.Sha3_384, targetPath)
 }
 
 // download writes an http.Request showing a progress.Meter
@@ -1419,7 +1582,7 @@ var download = func(ctx context.Context, name, sha3_384, downloadURL string, use
 		}
 
 		if pbar == nil {
-			pbar = &progress.NullProgress{}
+			pbar = progress.Null
 		}
 		pbar.Start(name, float64(resp.ContentLength))
 		mw := io.MultiWriter(w, h, pbar)
@@ -1575,13 +1738,9 @@ type assertionSvcError struct {
 
 // Assertion retrivies the assertion for the given type and primary key.
 func (s *Store) Assertion(assertType *asserts.AssertionType, primaryKey []string, user *auth.UserState) (asserts.Assertion, error) {
-	u, err := s.assertionsURI.Parse(path.Join(assertType.Name, path.Join(primaryKey...)))
-	if err != nil {
-		return nil, err
-	}
 	v := url.Values{}
 	v.Set("max-format", strconv.Itoa(assertType.MaxSupportedFormat()))
-	u.RawQuery = v.Encode()
+	u := s.assertionsEndpointURL(path.Join(assertType.Name, path.Join(primaryKey...)), v)
 
 	reqOptions := &requestOptions{
 		Method: "GET",
@@ -1608,7 +1767,12 @@ func (s *Store) Assertion(assertType *asserts.AssertionType, primaryKey []string
 					return fmt.Errorf("cannot decode assertion service error with HTTP status code %d: %v", resp.StatusCode, e)
 				}
 				if svcErr.Status == 404 {
-					return &AssertionNotFoundError{&asserts.Ref{Type: assertType, PrimaryKey: primaryKey}}
+					// best-effort
+					headers, _ := asserts.HeadersFromPrimaryKey(assertType, primaryKey)
+					return &asserts.NotFoundError{
+						Type:    assertType,
+						Headers: headers,
+					}
 				}
 				return fmt.Errorf("assertion service error: [%s] %q", svcErr.Title, svcErr.Detail)
 			}
@@ -1717,7 +1881,7 @@ func (s *Store) Buy(options *BuyOptions, user *auth.UserState) (*BuyResult, erro
 
 	reqOptions := &requestOptions{
 		Method:      "POST",
-		URL:         s.buyURI,
+		URL:         s.endpointURL(buyEndpPath, nil),
 		Accept:      jsonContentType,
 		ContentType: jsonContentType,
 		Data:        jsonData,
@@ -1781,7 +1945,7 @@ func (s *Store) ReadyToBuy(user *auth.UserState) error {
 
 	reqOptions := &requestOptions{
 		Method: "GET",
-		URL:    s.customersMeURI,
+		URL:    s.endpointURL(customersMeEndpPath, nil),
 		Accept: jsonContentType,
 	}
 
@@ -1813,3 +1977,16 @@ func (s *Store) ReadyToBuy(user *auth.UserState) error {
 		return &errors
 	}
 }
+
+func (s *Store) CacheDownloads() int {
+	return s.cfg.CacheDownloads
+}
+
+func (s *Store) SetCacheDownloads(fileCount int) {
+	s.cfg.CacheDownloads = fileCount
+	if fileCount > 0 {
+		s.cacher = NewCacheManager(dirs.SnapDownloadCacheDir, fileCount)
+	} else {
+		s.cacher = &nullCache{}
+	}
+}
