@@ -8,6 +8,8 @@ set -eux
 . "$TESTSLIB/snaps.sh"
 # shellcheck source=tests/lib/pkgdb.sh
 . "$TESTSLIB/pkgdb.sh"
+# shellcheck source=tests/lib/boot.sh
+. "$TESTSLIB/boot.sh"
 
 disable_kernel_rate_limiting() {
     # kernel rate limiting hinders debugging security policy so turn it off
@@ -18,6 +20,54 @@ disable_kernel_rate_limiting() {
     #sysctl -w kernel.printk_ratelimit=0
 }
 
+disable_refreshes() {
+    echo "Ensure jq is available"
+    if ! which jq; then
+        snap install --devmode jq
+    fi
+
+    echo "Modify state to make it look like the last refresh just happened"
+    systemctl stop snapd.socket snapd.service
+    jq ".data[\"last-refresh\"] = \"$(date +%Y-%m-%dT%H:%M:%S%:z)\"" /var/lib/snapd/state.json > /var/lib/snapd/state.json.new
+    mv /var/lib/snapd/state.json.new /var/lib/snapd/state.json
+    systemctl start snapd.socket snapd.service
+
+    echo "Minimize risk of hitting refresh schedule"
+    snap set core refresh.schedule=00:00-23:59
+    snap refresh --time|MATCH "last: 2[0-9]{3}"
+
+    echo "Ensure jq is gone"
+    snap remove jq
+}
+
+setup_systemd_snapd_overrides() {
+    START_LIMIT_INTERVAL="StartLimitInterval=0"
+    if [[ "$SPREAD_SYSTEM" = opensuse-42.2-* ]]; then
+        # StartLimitInterval is not supported by the systemd version
+        # openSUSE 42.2 ships.
+        START_LIMIT_INTERVAL=""
+    fi
+
+    mkdir -p /etc/systemd/system/snapd.service.d
+    cat <<EOF > /etc/systemd/system/snapd.service.d/local.conf
+[Unit]
+$START_LIMIT_INTERVAL
+[Service]
+Environment=SNAPD_DEBUG_HTTP=7 SNAPD_DEBUG=1 SNAPPY_TESTING=1 SNAPD_CONFIGURE_HOOK_TIMEOUT=30s SNAPPY_USE_STAGING_STORE=$SNAPPY_USE_STAGING_STORE
+ExecStartPre=/bin/touch /dev/iio:device0
+EOF
+    mkdir -p /etc/systemd/system/snapd.socket.d
+    cat <<EOF > /etc/systemd/system/snapd.socket.d/local.conf
+[Unit]
+$START_LIMIT_INTERVAL
+EOF
+
+    # We change the service configuration so reload and restart
+    # the snapd socket unit to get them applied
+    systemctl daemon-reload
+    systemctl restart snapd.socket
+}
+
 update_core_snap_for_classic_reexec() {
     # it is possible to disable this to test that snapd (the deb) works
     # fine with whatever is in the core snap
@@ -32,7 +82,7 @@ update_core_snap_for_classic_reexec() {
     # shove the new snap-exec and snapctl in there, and repack it.
 
     # First of all, unmount the core
-    core="$(readlink -f "$SNAPMOUNTDIR/core/current" || readlink -f "$SNAPMOUNTDIR/ubuntu-core/current")"
+    core="$(readlink -f "$SNAP_MOUNT_DIR/core/current" || readlink -f "$SNAP_MOUNT_DIR/ubuntu-core/current")"
     snap="$(mount | grep " $core" | awk '{print $1}')"
     umount --verbose "$core"
 
@@ -56,6 +106,13 @@ update_core_snap_for_classic_reexec() {
             ;;
     esac
 
+    case "$SPREAD_SYSTEM" in
+        ubuntu-*)
+            # also load snap-confine's apparmor profile
+            apparmor_parser -r squashfs-root/etc/apparmor.d/usr.lib.snapd.snap-confine.real
+            ;;
+    esac
+
     # repack, cheating to speed things up (4sec vs 1.5min)
     mv "$snap" "${snap}.orig"
     mksnap_fast "squashfs-root" "$snap"
@@ -73,7 +130,7 @@ update_core_snap_for_classic_reexec() {
     }
 
     # Make sure we're running with the correct copied bits
-    for p in "$LIBEXECDIR/snapd/snap-exec" "$LIBEXECDIR/snapd/snap-confine" "$LIBEXECDIR/snapd/snap-discard-ns" "$LIBEXECDIR/snapd/snapd"; do
+    for p in "$LIBEXECDIR/snapd/snap-exec" "$LIBEXECDIR/snapd/snap-confine" "$LIBEXECDIR/snapd/snap-discard-ns" "$LIBEXECDIR/snapd/snapd" "$LIBEXECDIR/snapd/snap-update-ns"; do
         check_file "$p" "$core/usr/lib/snapd/$(basename "$p")"
     done
     for p in /usr/bin/snapctl /usr/bin/snap; do
@@ -91,6 +148,11 @@ prepare_each_classic() {
 Environment=SNAP_REEXEC=$SNAP_REEXEC
 EOF
     fi
+    # the re-exec setting may have changed in the service so we need
+    # to ensure snapd is reloaded
+    systemctl daemon-reload
+    systemctl restart snapd
+
     if [ ! -f /etc/systemd/system/snapd.service.d/local.conf ]; then
         echo "/etc/systemd/system/snapd.service.d/local.conf vanished!"
         exit 1
@@ -119,39 +181,37 @@ prepare_classic() {
         exit 1
     fi
 
-    START_LIMIT_INTERVAL="StartLimitInterval=0"
-    if [[ "$SPREAD_SYSTEM" = opensuse-42.2-* ]]; then
-        # StartLimitInterval is not supported by the systemd version
-        # openSUSE 42.2 ships.
-        START_LIMIT_INTERVAL=""
-    fi
-
-    mkdir -p /etc/systemd/system/snapd.service.d
-    cat <<EOF > /etc/systemd/system/snapd.service.d/local.conf
-[Unit]
-$START_LIMIT_INTERVAL
-[Service]
-Environment=SNAPD_DEBUG_HTTP=7 SNAPD_DEBUG=1 SNAPPY_TESTING=1 SNAPD_CONFIGURE_HOOK_TIMEOUT=30s
-EOF
-    mkdir -p /etc/systemd/system/snapd.socket.d
-    cat <<EOF > /etc/systemd/system/snapd.socket.d/local.conf
-[Unit]
-$START_LIMIT_INTERVAL
-EOF
-
-    # We change the service configuration so reload and restart
-    # the snapd socket unit to get them applied
-    systemctl daemon-reload
-    systemctl restart snapd.socket
+    setup_systemd_snapd_overrides
 
     if [ "$REMOTE_STORE" = staging ]; then
         # shellcheck source=tests/lib/store.sh
         . "$TESTSLIB/store.sh"
+        # reset seeding data that is likely tainted with production keys
+        systemctl stop snapd.service snapd.socket
+        rm -rf /var/lib/snapd/assertions/*
+        rm -f /var/lib/snapd/state.json
         setup_staging_store
     fi
 
     # Snapshot the state including core.
     if [ ! -f "$SPREAD_PATH/snapd-state.tar.gz" ]; then
+        # Pre-cache a few heavy snaps so that they can be installed by tests
+        # quickly. This relies on a behavior of snapd where .partial files are
+        # used for resuming downloads.
+        (
+            set -x
+            cd "$TESTSLIB/cache/"
+            # Download each of the snaps we want to pre-cache. Note that `snap download`
+            # a quick no-op if the file is complete.
+            for snap_name in ${PRE_CACHE_SNAPS:-}; do
+                snap download "$snap_name"
+            done
+            # Copy all of the snaps back to the spool directory. From there we
+            # will reuse them during subsequent `snap install` operations.
+            cp *.snap /var/lib/snapd/snaps/
+            set +x
+        )
+
         ! snap list | grep core || exit 1
         # use parameterized core channel (defaults to edge) instead
         # of a fixed one and close to stable in order to detect defects
@@ -163,38 +223,29 @@ EOF
         update_core_snap_for_classic_reexec
         systemctl start snapd.{service,socket}
 
-        # ensure no auto-refresh happens during the tests
-        if [ -e /snap/core/current/meta/hooks/configure ]; then
-            snap set core refresh.schedule="$(date +%a --date=2days)@12:00-14:00"
-            snap set core refresh.disabled=true
-        fi
+        disable_refreshes
 
-        GRUB_EDITENV=grub-editenv
-        case "$SPREAD_SYSTEM" in
-            fedora-*|opensuse-*)
-                GRUB_EDITENV=grub2-editenv
-                ;;
-        esac
-
-        echo "Ensure that the grub-editenv list output does not contain any of the snap_* variables on classic"
-        output=$($GRUB_EDITENV list)
+        echo "Ensure that the bootloader environment output does not contain any of the snap_* variables on classic"
+        # shellcheck disable=SC2119
+        output=$(bootenv)
         if echo "$output" | MATCH snap_ ; then
-            echo "Expected grub environment without snap_*, got:"
+            echo "Expected bootloader environment without snap_*, got:"
             echo "$output"
             exit 1
         fi
 
         systemctl stop snapd.{service,socket}
         systemctl daemon-reload
-        escaped_snap_mount_dir="$(systemd-escape --path "$SNAPMOUNTDIR")"
+        escaped_snap_mount_dir="$(systemd-escape --path "$SNAP_MOUNT_DIR")"
         units="$(systemctl list-unit-files --full | grep -e "^$escaped_snap_mount_dir[-.].*\.mount" -e "^$escaped_snap_mount_dir[-.].*\.service" | cut -f1 -d ' ')"
         for unit in $units; do
             systemctl stop "$unit"
         done
         snapd_env="/etc/environment /etc/systemd/system/snapd.service.d /etc/systemd/system/snapd.socket.d"
-        tar czf "$SPREAD_PATH"/snapd-state.tar.gz /var/lib/snapd "$SNAPMOUNTDIR" /etc/systemd/system/"$escaped_snap_mount_dir"-*core*.mount $snapd_env
+        # shellcheck disable=SC2086
+        tar czf "$SPREAD_PATH"/snapd-state.tar.gz /var/lib/snapd "$SNAP_MOUNT_DIR" /etc/systemd/system/"$escaped_snap_mount_dir"-*core*.mount $snapd_env
         systemctl daemon-reload # Workaround for http://paste.ubuntu.com/17735820/
-        core="$(readlink -f "$SNAPMOUNTDIR/core/current")"
+        core="$(readlink -f "$SNAP_MOUNT_DIR/core/current")"
         # on 14.04 it is possible that the core snap is still mounted at this point, unmount
         # to prevent errors starting the mount unit
         if [[ "$SPREAD_SYSTEM" = ubuntu-14.04-* ]] && mount | grep -q "$core"; then
@@ -208,13 +259,11 @@ EOF
 
     if [[ "$SPREAD_SYSTEM" == debian-* || "$SPREAD_SYSTEM" == ubuntu-* ]]; then
         if [[ "$SPREAD_SYSTEM" == ubuntu-* ]]; then
-            quiet apt install -y -q pollinate
             pollinate
         fi
 
         # Improve entropy for the whole system quite a lot to get fast
         # key generation during our test cycles
-        apt-get install -y -q rng-tools
         echo "HRNGDEVICE=/dev/urandom" > /etc/default/rng-tools
         /etc/init.d/rng-tools restart
 
@@ -259,30 +308,6 @@ setup_reflash_magic() {
         # FIXME: netplan workaround
         mkdir -p "$UNPACKD/etc/netplan"
 
-        # set root pw by concating root line from host and rest from core
-        want_pw="$(grep ^root /etc/shadow)"
-        echo "$want_pw" > /tmp/new-shadow
-        tail -n +2 /etc/shadow >> /tmp/new-shadow
-        cp -v /tmp/new-shadow "$UNPACKD/etc/shadow"
-        cp -v /etc/passwd "$UNPACKD/etc/passwd"
-
-        # ensure spread -reuse works in the core image as well
-        if [ -e /.spread.yaml ]; then
-            cp -av /.spread.yaml "$UNPACKD"
-        fi
-
-        # we need the test user in the image
-        # see the comment in spread.yaml about 12345
-        sed -i 's/^test.*$//' "$UNPACKD"/etc/{shadow,passwd}
-        chroot "$UNPACKD" addgroup --quiet --gid 12345 test
-        chroot "$UNPACKD" adduser --quiet --no-create-home --uid 12345 --gid 12345 --disabled-password --gecos '' test
-        echo 'test ALL=(ALL) NOPASSWD:ALL' >> "$UNPACKD/etc/sudoers.d/99-test-user"
-
-        echo 'ubuntu ALL=(ALL) NOPASSWD:ALL' >> "$UNPACKD/etc/sudoers.d/99-ubuntu-user"
-
-        # modify sshd so that we can connect as root
-        sed -i 's/\(PermitRootLogin\|PasswordAuthentication\)\>.*/\1 yes/' "$UNPACKD/etc/ssh/sshd_config"
-
         # FIXME: install would be better but we don't have dpkg on
         #        the image
         # unpack our freshly build snapd into the new core snap
@@ -303,7 +328,7 @@ slots:
 EOF
 
         # build new core snap for the image
-        snapbuild "$UNPACKD" "$IMAGE_HOME"
+        snap pack "$UNPACKD" "$IMAGE_HOME"
 
         # FIXME: fetch directly once its in the assertion service
         cp "$TESTSLIB/assertions/pc-${REMOTE_STORE}.model" "$IMAGE_HOME/pc.model"
@@ -345,8 +370,64 @@ EOF
         mkdir -p /mnt/user-data/
         cp -ar /home/gopath /mnt/user-data/
 
-        # create test user home dir
+        # create test user and ubuntu user inside the writable partition
+        # so that we can use a stock core in tests
         mkdir -p /mnt/user-data/test
+
+        # create test user, see the comment in spread.yaml about 12345
+        mkdir -p /mnt/system-data/etc/sudoers.d/
+        echo 'test ALL=(ALL) NOPASSWD:ALL' >> /mnt/system-data/etc/sudoers.d/99-test-user
+        echo 'ubuntu ALL=(ALL) NOPASSWD:ALL' >> /mnt/system-data/etc/sudoers.d/99-ubuntu-user
+        # modify sshd so that we can connect as root
+        mkdir -p /mnt/system-data/etc/ssh
+        cp -a "$UNPACKD/etc/ssh/sshd_config" /mnt/system-data/etc/ssh/
+        sed -i 's/\(PermitRootLogin\|PasswordAuthentication\)\>.*/\1 yes/' /mnt/system-data/etc/ssh/sshd_config
+
+        # build the user database - this is complicated because:
+        # - spread on linode wants to login as "root"
+        # - "root" login on the stock core snap is disabled
+        # - we need to add our ubuntu and test users too
+        # - uids between classic/core differ
+        # - passwd,shadow on core are read-only
+        # - we cannot add root to extrausers as system passwd is searched first
+        # So we do:
+        # - take core passwd without "root" as extrausers
+        # - append root,ubuntu,test to extrausers
+        # - bind mount extrausers to /etc via custom systemd job
+        mkdir -p /mnt/system-data/var/lib/extrausers/
+        touch /mnt/system-data/var/lib/extrausers/sub{uid,gid}
+        mkdir -p /mnt/system-data/etc/systemd/system/multi-user.target.wants
+        for f in group gshadow passwd shadow; do
+            # the passwd from core without root
+            tail -n +2 "$UNPACKD/etc/$f" > /mnt/system-data/var/lib/extrausers/$f
+            # append this systems root user so that linode can connect
+            head -n1 /etc/$f >> /mnt/system-data/var/lib/extrausers/$f
+            # append ubuntu, test user for the testing
+            tail -n2 /etc/$f >> /mnt/system-data/var/lib/extrausers/$f
+
+            # now bind mount those passwd files on boot
+            cat <<EOF > /mnt/system-data/etc/systemd/system/etc-$f.mount
+[Unit]
+Description=Mount extrausers $f over system $f
+Before=ssh.service
+
+[Mount]
+What=/var/lib/extrausers/$f
+Where=/etc/$f
+Type=none
+Options=bind
+
+[Install]
+WantedBy=multi-user.target
+EOF
+            ln -s /etc/systemd/system/etc-$f.mount /mnt/system-data/etc/systemd/system/multi-user.target.wants/etc-$f.mount
+        done
+
+        # ensure spread -reuse works in the core image as well
+        if [ -e /.spread.yaml ]; then
+            cp -av /.spread.yaml /mnt/system-data
+        fi
+
         # using symbolic names requires test:test have the same ids
         # inside and outside which is a pain (see 12345 above), but
         # using the ids directly is the wrong kind of fragile
@@ -359,20 +440,6 @@ EOF
         (cd /tmp ; unsquashfs -v "$IMAGE_HOME"/core_*.snap etc/systemd/system)
         cp -avr /tmp/squashfs-root/etc/systemd/system /mnt/system-data/etc/systemd/
 
-        # FIXUP silly systemd
-        mkdir -p /mnt/system-data/etc/systemd/system/snapd.service.d
-        cat <<EOF > /mnt/system-data/etc/systemd/system/snapd.service.d/local.conf
-[Unit]
-StartLimitInterval=0
-[Service]
-Environment=SNAPD_DEBUG_HTTP=7 SNAPD_DEBUG=1 SNAPPY_TESTING=1 SNAPPY_USE_STAGING_STORE=$SNAPPY_USE_STAGING_STORE
-ExecStartPre=/bin/touch /dev/iio:device0
-EOF
-        mkdir -p /mnt/system-data/etc/systemd/system/snapd.socket.d
-        cat <<EOF > /mnt/system-data/etc/systemd/system/snapd.socket.d/local.conf
-[Unit]
-StartLimitInterval=0
-EOF
 
         umount /mnt
         kpartx -d  "$IMAGE_HOME/$IMAGE"
@@ -440,11 +507,8 @@ prepare_all_snap() {
         fi
     done
 
-    # ensure no auto-refresh happens during the tests
-    if [ -e /snap/core/current/meta/hooks/configure ]; then
-        snap set core refresh.schedule="$(date +%a --date=2days)@12:00-14:00"
-        snap set core refresh.disabled=true
-    fi
+    disable_refreshes
+    setup_systemd_snapd_overrides
 
     # Snapshot the fresh state (including boot/bootenv)
     if [ ! -f "$SPREAD_PATH/snapd-state.tar.gz" ]; then
